@@ -87,28 +87,13 @@
 
 ;;;; basic LAYOUT stuff
 
-;;; Note: This bound is set somewhat less than MOST-POSITIVE-FIXNUM
-;;; in order to guarantee that several hash values can be added without
-;;; overflowing into a bignum.
-(defconstant layout-clos-hash-limit (1+ (ash sb-xc:most-positive-fixnum -3))
-  "the exclusive upper bound on LAYOUT-CLOS-HASH values")
-;; This must be DEF!TYPE and not just DEFTYPE because access to slots
-;; of a layout occur "before" the structure definition is made in the
-;; run-the-xc pass, and the source-transform of a slot accessor
-;; wraps (TRULY-THE <type> ...) around %INSTANCE-REF,
-;; so <type> had best be defined at that point.
-(def!type layout-clos-hash () `(integer 0 ,layout-clos-hash-limit))
-(declaim (ftype (sfunction (t) layout-clos-hash) randomish-layout-clos-hash))
-
 ;;; Careful here: if you add more bits, then adjust the bit packing for
 ;;; 64-bit layouts which also store LENGTH + DEPTHOID in the same word.
 (defconstant +structure-layout-flag+         #b00000001)
 (defconstant +pathname-layout-flag+          #b00000010)
-(defconstant +ctype-layout-flag+             #b00000100)
 (defconstant +condition-layout-flag+         #b00001000)
 (defconstant +pcl-object-layout-flag+        #b00010000)
 (defconstant sb-vm:lockfree-list-node-flag   #b01000000) ; exported for use in gc-private.h
-(defconstant sb-vm:layout-layout-flag        #b10000000) ; "
 
 ;;; The LAYOUT structure is pointed to by the first cell of instance
 ;;; (or structure) objects. It represents what we need to know for
@@ -150,9 +135,9 @@
 (sb-xc:defstruct (layout
              ;; Accept a specific subset of keywords
              #+64-bit (:constructor %make-layout
-                          (clos-hash classoid flags inherits info bitmap))
+                          (clos-hash classoid flags info bitmap))
              #-64-bit (:constructor %make-layout
-                          (clos-hash classoid depthoid length flags inherits info bitmap))
+                          (clos-hash classoid depthoid length flags info bitmap))
              (:copier nil))
 
   ;; A packed field containing the DEPTHOID, LENGTH, and FLAGS
@@ -161,8 +146,10 @@
   ;; a union of +something-LAYOUT-FLAG+ bits
   #-64-bit (flags 0 :type word :read-only nil)
 
-  ;; a pseudo-random hash value for use by CLOS.
-  (clos-hash (missing-arg) :type layout-clos-hash) ; this no longer defaults to a random number
+  ;; a quasi-random hash value for use by CLOS. Determine by class-name
+  ;; for classes named by a symbol, otherwise a pseudo-random value.
+  ;; Must be acceptable as an argument to SB-INT:MIX
+  (clos-hash (missing-arg) :type (and fixnum unsigned-byte))
   ;; the class that this is a layout for
   (classoid (missing-arg) :type classoid)
   ;; The value of this slot can be:
@@ -228,16 +215,40 @@
            (ash ,(or length 0) 16)
            ,(or flags 0)))
 
+(defmacro set-layout-inherits (layout inherits &optional depthoid)
+  `(let* ((l ,layout) (i ,inherits) (d ,(or depthoid '(length i))))
+     (declare (ignorable i d))
+     ;; I tried putting a /SHOW here for debugging, but it's just too broken
+     ;; because layouts affect the printer dispatch mechanism.
+     #+nil
+     (let ((*print-pretty* nil))
+       (fresh-line)
+       (write-string "SET-INHERITS ")
+       (write (layout-classoid-name l))
+       (write-string " ")
+       (write (map 'list #'layout-classoid-name i))
+       (terpri))
+     (setf (layout-inherits l) i)
+     #-sb-xc-host
+     (setf (layout-ancestor_2 l) (if (> d 2) (svref i 2) 0)
+           (layout-ancestor_3 l) (if (> d 3) (svref i 3) 0)
+           (layout-ancestor_4 l) (if (> d 4) (svref i 4) 0)
+           (layout-ancestor_5 l) (if (> d 5) (svref i 5) 0))
+     l))
+
 #-sb-xc-host
 (defun make-layout (clos-hash classoid
                     &key (depthoid -1) (length 0) (flags 0)
-                         (inherits #())
+                         (inherits #() inheritsp)
                          (info nil)
                          (bitmap (if info (dd-bitmap info) +layout-all-tagged+)))
-  (%make-layout clos-hash classoid
-                #+64-bit (pack-layout-flags depthoid length flags)
-                #-64-bit depthoid #-64-bit length #-64-bit flags
-                inherits info bitmap))
+  (let ((layout (%make-layout clos-hash classoid
+                              #+64-bit (pack-layout-flags depthoid length flags)
+                              #-64-bit depthoid #-64-bit length #-64-bit flags
+                              info bitmap)))
+    (when inheritsp
+      (set-layout-inherits layout inherits))
+    layout))
 
 ;;; The cross-compiler representation of a LAYOUT omits several things:
 ;;;   * BITMAP - obtainable via (DD-MAPMAP (LAYOUT-INFO layout)).
@@ -253,8 +264,12 @@
 (progn
   (defstruct (layout (:include structure!object)
                      (:constructor make-layout
-                         (clos-hash classoid &key depthoid length flags inherits info)))
-    (clos-hash nil :type layout-clos-hash) ; needed for cross-compiling some TYPECASE forms
+                                   (clos-hash classoid &key depthoid length flags inherits info)))
+    ;; CLOS-HASH is needed to convert some TYPECASE forms to jump tables.
+    ;; Theoretically we don't need this in the cross-compiler, because the
+    ;; layout has a classoid which has a name which has a known hash.
+    ;; But there's no harm in storing it.
+    (clos-hash nil :type (and sb-xc:fixnum unsigned-byte))
     (classoid nil :type classoid)
     (flags 0 :type word)
     (invalid :uninitialized :type (or cons (member nil t :uninitialized)))
@@ -265,15 +280,6 @@
   (defun layout-bitmap (layout)
     (if (layout-info layout) (dd-bitmap (layout-info layout)) +layout-all-tagged+)))
 
-(defmacro populate-layout-ancestors (layout inherits &optional depthoid)
-  `(let* ((l ,layout) (i ,inherits) (d ,(or depthoid '(length i))))
-     (declare (ignorable i d))
-     #-sb-xc-host
-     (setf (layout-ancestor_2 l) (if (> d 2) (svref i 2) 0)
-           (layout-ancestor_3 l) (if (> d 3) (svref i 3) 0)
-           (layout-ancestor_4 l) (if (> d 4) (svref i 4) 0)
-           (layout-ancestor_5 l) (if (> d 5) (svref i 5) 0))
-     l))
 (defmacro sb-c::layout-nth-ancestor-slot (n)
   `(case ,n
      (2 'layout-ancestor_2)
@@ -307,13 +313,15 @@
                (layout-length ,layout) ,length
                (layout-flags ,layout) ,flags))))
 
-(declaim (inline layout-for-std-class-p))
-(defun layout-for-std-class-p (x)
+;;; True of STANDARD-OBJECT, which include generic functions.
+;;; This one includes any class that mixes in STANDARD-OBJECT.
+(declaim (inline layout-for-pcl-obj-p))
+(defun layout-for-pcl-obj-p (x)
   (logtest (layout-flags x) +pcl-object-layout-flag+))
 
 (declaim (inline sb-fasl:dumpable-layout-p))
 (defun sb-fasl:dumpable-layout-p (x)
-  (and (typep x 'layout) (not (layout-for-std-class-p x))))
+  (and (typep x 'layout) (not (layout-for-pcl-obj-p x))))
 
 ;;; The CLASSOID structure is a supertype of all classoid types.  A
 ;;; CLASSOID is also a CTYPE structure as recognized by the type
