@@ -31,46 +31,11 @@
   (def))
 
 ;;; Is X an extended sequence?
-;; This is like the "hierarchical layout depths for other things"
-;; case of the TYPEP transform (cf 'typetran'). Ideally it would
-;; be preferable to share TYPEP's code rather than repeat it here.
 (declaim (maybe-inline extended-sequence-p))
-(defun extended-sequence-p (x)
-  (let* ((slayout #.(info :type :compiler-layout 'sequence))
-         (depthoid #.(layout-depthoid (info :type :compiler-layout 'sequence)))
-         (layout
-          ;; It is not an error to define a class that is both SEQUENCE and
-          ;; FUNCALLABLE-INSTANCE with metaclass FUNCALLABLE-STANDARD-CLASS
-          (cond ((%instancep x) (%instance-layout x))
-                ((funcallable-instance-p x) (%funcallable-instance-layout x))
-                (t (return-from extended-sequence-p nil)))))
-    (when (layout-invalid layout)
-      (setq layout (update-object-layout-or-invalid x slayout)))
-    ;; It's _nearly_ impossible to create an instance which is exactly
-    ;; of type SEQUENCE. To wit: (make-instance 'sequence) =>
-    ;;   "Cannot allocate an instance of #<BUILT-IN-CLASS SEQUENCE>."
-    ;; We should not need to check for that, just the 'inherits' vector.
-    ;; However, bootstrap code does a sleazy thing, making an instance of
-    ;; the abstract base type which is impossible for user code to do.
-    ;; Preferably the prototype instance for SEQUENCE would be one that could
-    ;; exist, so it would be a STANDARD-OBJECT and SEQUENCE. But it's not.
-    ;; Hence we have to check for a layout that no code using the documented
-    ;; sequence API would ever see, just to get the boundary case right.
-    ;; Note also:
-    ;; - Some builtins use a prototype object that is strictly deeper than
-    ;;   layout of the named class because it is indeed the case that no
-    ;;   object's layout can ever be EQ to that of the ancestor.
-    ;;   e.g. a fixnum as representative of class REAL
-    ;; - Some builtins actually fail (TYPEP (CLASS-PROTOTYPE X) X)
-    ;;   but that's not an excuse for getting SEQUENCE wrong:
-    ;;    (CLASS-PROTOTYPE (FIND-CLASS 'FLOAT)) => 42
-    ;;    (CLASS-PROTOTYPE (FIND-CLASS 'VECTOR)) => 42
-    ;;    (CLASS-PROTOTYPE (FIND-CLASS 'LIST)) => 42
-    ;;    (CLASS-PROTOTYPE (FIND-CLASS 'STRING)) => 42
-    (let ((inherits (layout-inherits (truly-the layout layout))))
-      (declare (optimize (safety 0)))
-      (eq (if (> (length inherits) depthoid) (svref inherits depthoid) layout)
-          slayout))))
+(defun extended-sequence-p (sb-c::object) ; name the argugment as required by transform
+  (macrolet ((transform ()
+               (sb-c::transform-instance-typep (find-classoid 'sequence))))
+    (transform)))
 
 ;;; Is X a SEQUENCE?  Harder than just (OR VECTOR LIST)
 (defun sequencep (x)
@@ -182,30 +147,52 @@
   (and (fixnump x)
        (<= 0 x limit)))
 
+;;; a vector that maps widetags to layouts, used for quickly finding
+;;; the layouts of built-in classes
+(define-load-time-global **primitive-object-layouts** nil)
+(declaim (type simple-vector **primitive-object-layouts**))
+(defun !pred-cold-init ()
+  ;; This vector is allocated in immobile space when possible. There isn't
+  ;; a way to do that from lisp, so it's special-cased in genesis.
+  #-immobile-space (setq **primitive-object-layouts** (make-array 256))
+  (map-into **primitive-object-layouts**
+            (lambda (name) (classoid-layout (find-classoid name)))
+            #.(let ((table (make-array 256 :initial-element 'sb-kernel::random-class)))
+                (dolist (x sb-kernel::+!built-in-classes+)
+                  (destructuring-bind (name &key codes &allow-other-keys) x
+                    (dolist (code codes)
+                      (setf (svref table code) name))))
+                (loop for i from sb-vm:list-pointer-lowtag by (* 2 sb-vm:n-word-bytes)
+                      below 256
+                      do (setf (aref table i) 'cons))
+                (loop for i from sb-vm:even-fixnum-lowtag by (ash 1 sb-vm:n-fixnum-tag-bits)
+                      below 256
+                      do (setf (aref table i) 'fixnum))
+                table)))
+
 ;;; Return the layout for an object. This is the basic operation for
 ;;; finding out the "type" of an object, and is used for generic
 ;;; function dispatch. The standard doesn't seem to say as much as it
 ;;; should about what this returns for built-in objects. For example,
 ;;; it seems that we must return NULL rather than LIST when X is NIL
 ;;; so that GF's can specialize on NULL.
+;;; x86-64 has a vop that implements this without even needing to place
+;;; the vector of layouts in the constant pool of the containing code.
+#-(and compact-instance-header x86-64)
+(progn
 (declaim (inline layout-of))
-#-sb-xc-host
 (defun layout-of (x)
   (declare (optimize (speed 3) (safety 0)))
-  #+(and compact-instance-header x86-64)
-  (values (%primitive layout-of x
-                      (load-time-value sb-kernel::**built-in-class-codes** t)))
-  #-(and compact-instance-header x86-64)
   (cond ((%instancep x) (%instance-layout x))
-        ((funcallable-instance-p x) (%funcallable-instance-layout x))
+        ((funcallable-instance-p x) (%fun-layout x))
         ;; Compiler can dump literal layouts, which handily sidesteps
         ;; the question of when cold-init runs L-T-V forms.
         ((null x) #.(find-layout 'null))
         (t
          ;; Note that WIDETAG-OF is slightly suboptimal here and could be
          ;; improved - we've already ruled out some of the lowtags.
-         (svref (load-time-value sb-kernel::**built-in-class-codes** t)
-                (widetag-of x)))))
+         (svref (load-time-value **primitive-object-layouts** t)
+                (widetag-of x))))))
 
 (declaim (inline classoid-of))
 #-sb-xc-host
@@ -229,10 +216,10 @@
      (cond
        ((<= 0 object 1) 'bit)
        ((< object 0) 'fixnum)
-       (t '(integer 0 #.sb-xc:most-positive-fixnum))))
+       (t `(integer 0 ,sb-xc:most-positive-fixnum))))
     (integer
      (if (>= object 0)
-         '(integer #.(1+ sb-xc:most-positive-fixnum))
+         `(integer ,(1+ sb-xc:most-positive-fixnum))
          'bignum))
     (character
      (typecase object
@@ -278,20 +265,26 @@
 ;;; IR1 might potentially transform EQL into %EQL/INTEGER.
 #+integer-eql-vop
 (defun %eql/integer (obj1 obj2)
-  ;; This is just for constant folding, no need to transform into the %EQL/INTEGER VOP
-  (eql obj1 obj2))
+  ;; This is just for constant folding, there's no real need to transform
+  ;; into the %EQL/INTEGER VOP. But it's confusing if it becomes identical to
+  ;; EQL, and then due to ICF we find that #'EQL => #<FUNCTION %EQL/INTEGER>.
+  ;; Type declarations don't suffice because we don't know which arg is an integer
+  ;; (if not both). We could ensure selection of the vop by using %PRIMITIVE.
+  ;; Anyway it suffices to disable type checking and pretend its the always
+  ;; the first arg that's an integer, but that won't work on the host because
+  ;; it might enforce the type since we can't portably unenforce after declaring.
+  #-sb-xc-host (declare (optimize (sb-c::type-check 0)))
+  (eql #+sb-xc-host obj1 #-sb-xc-host (the integer obj1) obj2))
 
 (declaim (inline %eql))
 (defun %eql (obj1 obj2)
   "Return T if OBJ1 and OBJ2 represent the same object, otherwise NIL."
+  #+x86-64 (eql obj1 obj2) ; vop fully implements all cases of EQL
+  #-x86-64 ; else this is the only full implementation of EQL
   (or (eq obj1 obj2)
       (if (or (typep obj2 'fixnum)
               (not (typep obj2 'number)))
           nil
-          ;; I would think that we could do slightly better here by testing that
-          ;; both objs are OTHER-POINTER-P with equal %OTHER-POINTER-WIDETAGs.
-          ;; Then dispatch on obj2 and elide the TYPEP on obj1 using TRULY-THE.
-          ;; Also would need to deal with immediate single-float for 64-bit.
           (macrolet ((foo (&rest stuff)
                        `(typecase obj2
                           ,@(mapcar (lambda (foo)
@@ -327,7 +320,9 @@
                      (eql (imagpart x) (imagpart y))))))))))
 
 (defun eql (x y)
-  (%eql x y))
+  ;; On x86-64, EQL is just an interpreter stub for a vop.
+  ;; For others it's a call to the implementation of generic EQL.
+  (#+x86-64 eql #-x86-64 %eql x y))
 
 (defun bit-vector-= (x y)
   (declare (type bit-vector x y))
@@ -363,6 +358,11 @@ length and have identical components. Other arrays must be EQ to be EQUAL."
                          (equal-aux (cdr x) (cdr y))))
                    ((stringp x)
                     (and (stringp y) (string= x y)))
+                   ;; We could remove this case by ensuring that MAKE-PATHNAME,
+                   ;; PARSE-NAMESTRING, MERGE-PATHNAME, etc look in a weak hash-based
+                   ;; thing first for an EQUAL pathname, ensuring that if two pathnames
+                   ;; are EQUAL then they are EQ. That would elide this test at the
+                   ;; expense of pathname construction which seems like a good tradeoff.
                    ((pathnamep x)
                     (and (pathnamep y) (pathname= x y)))
                    ((bit-vector-p x)
@@ -402,36 +402,34 @@ length and have identical components. Other arrays must be EQ to be EQUAL."
                       x)
              t))))
 
+(declaim (inline instance-equalp))
 (defun instance-equalp (x y)
   (let ((layout-x (%instance-layout x)))
     (and
      (eq layout-x (%instance-layout y))
-     (logtest +structure-layout-flag+ (layout-%flags layout-x))
+     (logtest +structure-layout-flag+ (layout-flags layout-x))
      (macrolet ((slot-ref-equalp ()
                   `(let ((x-el (%instance-ref x i))
                          (y-el (%instance-ref y i)))
                      (or (eq x-el y-el) (equalp x-el y-el)))))
-         (if (eql (layout-bitmap layout-x) sb-kernel::+layout-all-tagged+)
-             (loop for i of-type index from sb-vm:instance-data-start
-                   below (layout-length layout-x)
+       (let ((n (%instance-length x)))
+         (if (eql (layout-bitmap layout-x) sb-kernel:+layout-all-tagged+)
+             (loop for i downfrom (1- n) to sb-vm:instance-data-start
                    always (slot-ref-equalp))
              (let ((comparators (layout-equalp-tests layout-x)))
-               (unless (= (length comparators)
-                          (- (layout-length layout-x) sb-vm:instance-data-start))
+               (unless (= (length comparators) (- n sb-vm:instance-data-start))
                  (bug "EQUALP got incomplete instance layout"))
                ;; See remark at the source code for %TARGET-DEFSTRUCT
                ;; explaining how to use the vector of comparators.
-               (loop for i of-type index from sb-vm:instance-data-start
-                     below (layout-length layout-x)
+               (loop for i downfrom (1- n) to sb-vm:instance-data-start
                      for test = (data-vector-ref
                                  comparators (- i sb-vm:instance-data-start))
                      always (cond ((eql test 0) (slot-ref-equalp))
-                                  ((functionp test)
-                                   (funcall test i x y))
-                                  (t)))))))))
+                                  ((functionp test) (funcall test i x y))
+                                  (t))))))))))
 
 ;;; Doesn't work on simple vectors
-(defun array-equal-p (x y)
+(defun array-equalp (x y)
   (declare (array x y))
   (let ((rank (array-rank x)))
     (and
@@ -475,13 +473,12 @@ length and have identical components. Other arrays must be EQ to be EQUAL."
                                 (equalp x-el y-el)))))))))
 
 (defun equalp (x y)
-  #+nil ; KLUDGE: If doc string, should be accurate: Talk about structures
-  ; and HASH-TABLEs.
-  "This is like EQUAL, except more liberal in several respects.
+  "Just like EQUAL, but more liberal in several respects.
   Numbers may be of different types, as long as the values are identical
   after coercion. Characters may differ in alphabetic case. Vectors and
   arrays must have identical dimensions and EQUALP elements, but may differ
-  in their type restriction."
+  in their type restriction. The elements of structured components
+  are compared with EQUALP."
   (cond ((eq x y) t)
         ((characterp x) (and (characterp y) (char-equal x y)))
         ((numberp x) (and (numberp y) (= x y)))
@@ -497,22 +494,32 @@ length and have identical components. Other arrays must be EQ to be EQUAL."
         ((%instancep x)
          (and (%instancep y)
               (instance-equalp x y)))
-        ((and (bit-vector-p x)
-              (bit-vector-p y))
+        ((and (simple-vector-p x) (simple-vector-p y))
+         (let ((len (length x)))
+           (and (= len (length y))
+                (loop for i below len ; somewhat faster than the generic loop
+                      always (let ((a (svref x i)) (b (svref y i)))
+                               (or (eq a b) (equalp a b)))))))
+        ((and (bit-vector-p x) (bit-vector-p y))
          (bit-vector-= x y))
         ((vectorp x)
          (and (vectorp y)
               (vector-equalp x y)))
         ((arrayp x)
          (and (arrayp y)
-              (array-equal-p x y)))
+              (array-equalp x y)))
         (t nil)))
 
-(let ((test-cases `((0.0 ,(load-time-value (make-unportable-float :single-float-negative-zero)) t)
-                    (0.0 1.0 nil)
-                    (#c(1 0) #c(1.0 0.0) t)
-                    (#c(0 1) #c(0.0 1.0) t)
-                    (#c(1.1 0.0) #c(11/10 0) nil) ; due to roundoff error
+(let ((test-cases `(($0.0 $-0.0 t)
+                    ($0.0 $1.0 nil)
+                    ;; There is no cross-compiler #C reader macro.
+                    ;; SB-XC:COMPLEX does not want uncanonical input, i.e. imagpart
+                    ;; of rational 0 which downgrades the result to just an integer.
+                    (1 ,(complex $1.0 $0.0) t)
+                    (,(complex 0 1) ,(complex $0.0 $1.0) t)
+                    ;; 11/10 is unequal to real 1.1 due to roundoff error.
+                    ;; COMPLEX here is a red herring
+                    (,(complex $1.1 $0.0) 11/10 nil)
                     ("Hello" "hello" t)
                     ("Hello" #(#\h #\E #\l #\l #\o) t)
                     ("Hello" "goodbye" nil))))
