@@ -67,7 +67,7 @@
       (setf (elt (character-attribute-array rt) (char-code char)) newvalue)
       (if (= newvalue +char-attr-constituent+)
           ;; Default value for the C-A-HASH-TABLE is +CHAR-ATTR-CONSTITUENT+.
-          (%remhash char (character-attribute-hash-table rt))
+          (remhash char (character-attribute-hash-table rt))
           (setf (gethash char (character-attribute-hash-table rt)) newvalue)))
   (values))
 
@@ -104,12 +104,12 @@
         (gethash sub-char it))))
 
 ;; Coerce THING to a character-macro-table entry
-(defmacro !coerce-to-cmt-entry (thing)
+(defmacro coerce-to-cmt-entry (thing)
   `(let ((x ,thing))
      (if (typep x '(or null function)) x (find-or-create-fdefn x))))
 
 ;; Return a callable function given a character-macro-table entry.
-(defmacro !cmt-entry-to-function (val fallback)
+(defmacro cmt-entry-to-function (val fallback)
   `(let ((x ,val))
      (truly-the
       function
@@ -118,7 +118,7 @@
             (t (sb-c:safe-fdefn-fun x))))))
 
 ;; Return a function-designator given a character-macro-table entry.
-(defmacro !cmt-entry-to-fun-designator (val)
+(defmacro cmt-entry-to-fun-designator (val)
   `(let ((x ,val))
      (if (fdefn-p x) (fdefn-name x) x)))
 
@@ -171,7 +171,8 @@
 ;;; FIXME: this initform is considered too hairy to assign (a constant array, really?)
 ;;; if changed to DEFCONSTANT-EQX, which makes this file unslammable as-is. Oh well.
 (defconstant +constituent-trait-table+
-  #.(let ((a (!make-specialized-array base-char-code-limit '(unsigned-byte 8))))
+  #.(let ((a (sb-xc:make-array base-char-code-limit
+                               :element-type '(unsigned-byte 8))))
       (fill a +char-attr-constituent+)
       (flet ((!set-constituent-trait (char trait)
                (aver (typep char 'base-char))
@@ -189,11 +190,13 @@
         (!set-constituent-trait #\D +char-attr-constituent-expt+)
         (!set-constituent-trait #\S +char-attr-constituent-expt+)
         (!set-constituent-trait #\L +char-attr-constituent-expt+)
+        (!set-constituent-trait #\R +char-attr-constituent-expt+) ; extension
         (!set-constituent-trait #\e +char-attr-constituent-expt+)
         (!set-constituent-trait #\f +char-attr-constituent-expt+)
         (!set-constituent-trait #\d +char-attr-constituent-expt+)
         (!set-constituent-trait #\s +char-attr-constituent-expt+)
         (!set-constituent-trait #\l +char-attr-constituent-expt+)
+        (!set-constituent-trait #\r +char-attr-constituent-expt+) ; extension
         (!set-constituent-trait #\Space +char-attr-invalid+)
         (!set-constituent-trait #\Newline +char-attr-invalid+)
         (dolist (c (list backspace-char-code tab-char-code form-feed-char-code
@@ -224,6 +227,7 @@
         (ecase case (:upcase 0) (:downcase 1) (:preserve 2) (:invert 3)))
   case)
 
+(declaim (inline readtable-normalization))
 (defun readtable-normalization (readtable)
   "Returns T if READTABLE normalizes strings to NFKC, and NIL otherwise.
 The READTABLE-NORMALIZATION of the standard readtable is T."
@@ -343,7 +347,7 @@ standard Lisp readtable when NIL."
                             +char-attr-constituent+
                             +char-attr-terminating-macro+)
                    designated-readtable)
-    (set-cmt-entry char (!coerce-to-cmt-entry function) designated-readtable)
+    (set-cmt-entry char (coerce-to-cmt-entry function) designated-readtable)
     t)) ; (ANSI-specified return value)
 
 (defun get-macro-character (char &optional (rt-designator *readtable*))
@@ -354,7 +358,7 @@ standard Lisp readtable when NIL."
   (let* ((designated-readtable (or rt-designator *standard-readtable*))
          ;; the first return value: (OR FUNCTION SYMBOL) if CHAR is a macro
          ;; character, or NIL otherwise
-         (fun-value (!cmt-entry-to-fun-designator
+         (fun-value (cmt-entry-to-fun-designator
                      (get-raw-cmt-entry char designated-readtable))))
     (values fun-value
             ;; NON-TERMINATING-P return value:
@@ -405,7 +409,7 @@ standard Lisp readtable when NIL."
     (when (digit-char-p sub-char)
       (error "SUB-CHAR must not be a decimal digit: ~S" sub-char))
     (let ((dtable (get-dispatch-macro-char-table disp-char readtable))
-          (function (!coerce-to-cmt-entry function)))
+          (function (coerce-to-cmt-entry function)))
       ;; (SET-MACRO-CHARACTER #\$ (GET-MACRO-CHARACTER #\#)) will share
       ;; the dispatch table. Perhaps it should be copy-on-write?
       (if (typep sub-char 'base-char)
@@ -426,7 +430,7 @@ standard Lisp readtable when NIL."
    or NIL if there is no associated function."
   (let ((dtable (get-dispatch-macro-char-table
                  disp-char (or rt-designator *standard-readtable*))))
-    (!cmt-entry-to-fun-designator
+    (cmt-entry-to-fun-designator
      (get-raw-cmt-dispatch-entry (char-upcase sub-char) dtable))))
 
 
@@ -550,7 +554,7 @@ standard Lisp readtable when NIL."
 
 ;; A list of available TOKEN-BUFs
 (declaim (type (or null token-buf) *token-buf-pool*))
-(!define-thread-local *token-buf-pool* nil)
+(define-thread-local *token-buf-pool* nil)
 
 (defun reset-read-buffer (buffer)
   ;; Turn BUFFER into an empty read buffer.
@@ -751,7 +755,7 @@ standard Lisp readtable when NIL."
     (and (form-tracking-stream-p stream)
          ;; Subtract 1 because the position points _after_ CHAR.
          (1- (form-tracking-stream-input-char-pos stream)))
-    (funcall (!cmt-entry-to-function
+    (funcall (cmt-entry-to-function
               (get-raw-cmt-entry char *readtable*) #'read-token)
              stream char)))
 
@@ -1185,20 +1189,27 @@ standard Lisp readtable when NIL."
 (declaim (type (or null package) *reader-package*)
          (always-bound *reader-package*))
 
-(defun reader-find-package (package-designator stream)
+(defun reader-find-package (package-designator stream restarts)
   (if (%instancep package-designator)
       package-designator
-      (let ((package (find-package package-designator)))
-        (cond (package
-               ;; Release the token-buf that was used for the designator
-               (release-token-buf (shiftf (token-buf-next *read-buffer*) nil))
-               package)
-              (t
-               (error 'simple-reader-package-error
-                      :package package-designator
-                      :stream stream
-                      :format-control "Package ~A does not exist."
-                      :format-arguments (list package-designator)))))))
+      (block nil
+        (tagbody retry
+           (let ((package (find-package package-designator)))
+             (cond (package
+                    ;; Release the token-buf that was used for the designator
+                    (release-token-buf (shiftf (token-buf-next *read-buffer*) nil))
+                    (return (values package nil)))
+                   (t
+                    (macrolet ((err ()
+                                 `(error 'simple-reader-package-error
+                                         :package package-designator
+                                         :stream stream
+                                         :format-control "Package ~A does not exist."
+                                         :format-arguments (list package-designator))))
+                      (if restarts
+                          (find-package-restarts (package-designator t)
+                            (err))
+                          (err))))))))))
 
 (defun read-token (stream firstchar)
   "Default readmacro function. Handles numbers, symbols, and SBCL's
@@ -1527,7 +1538,7 @@ extended <package-name>::<form-in-package> syntax."
          (unread-char char stream)
          (if package-designator
              (let* ((*reader-package*
-                     (reader-find-package package-designator stream)))
+                     (reader-find-package package-designator stream nil)))
                (return (read stream t nil t)))
              (simple-reader-error stream
                                   "illegal terminating character after a double-colon: ~S"
@@ -1540,33 +1551,38 @@ extended <package-name>::<form-in-package> syntax."
                               package-designator))
         (t (go SYMBOL)))
       RETURN-SYMBOL
-      (setf buf (normalize-read-buffer buf))
-      (casify-read-buffer buf)
-      (let* ((pkg (if package-designator
-                      (reader-find-package package-designator stream)
-                      (or *reader-package* (sane-package))))
-             (intern-p (or (/= colons 1) (eq pkg *keyword-package*))))
-        (unless intern-p ; Try %FIND-SYMBOL
-          (multiple-value-bind (symbol accessibility)
-              (%find-symbol (token-buf-string buf) (token-buf-fill-ptr buf) pkg)
-            (when (eq accessibility :external) (return symbol))
-            (with-simple-restart (continue "Use symbol anyway.")
-              (error 'simple-reader-package-error
-                     :package pkg
-                     :stream stream
-                     :format-arguments
-                     (list (copy-token-buf-string buf) (package-name pkg))
-                     :format-control
-                     (if accessibility
-                         "The symbol ~S is not external in the ~A package."
-                         "Symbol ~S not found in the ~A package.")))))
-        (return (%intern (token-buf-string buf)
-                         (token-buf-fill-ptr buf)
-                         pkg
-                         (if (token-buf-only-base-chars buf)
-                             (%readtable-symbol-preference rt)
-                             'character)
-                         nil)))))))
+        (setf buf (normalize-read-buffer buf))
+        (casify-read-buffer buf)
+        (multiple-value-bind (pkg restart-kind)
+            (if package-designator
+                (reader-find-package package-designator stream t)
+                (or *reader-package* (sane-package)))
+          (if (eq restart-kind :uninterned)
+              (return (make-symbol (copy-token-buf-string buf)))
+              (let* ((intern-p (or (/= colons 1)
+                                   (eq pkg *keyword-package*)
+                                   (eq restart-kind :current))))
+                (unless intern-p        ; Try %FIND-SYMBOL
+                  (multiple-value-bind (symbol accessibility)
+                      (%find-symbol (token-buf-string buf) (token-buf-fill-ptr buf) pkg)
+                    (when (eq accessibility :external) (return symbol))
+                    (with-simple-restart (continue "Use symbol anyway.")
+                      (error 'simple-reader-package-error
+                             :package pkg
+                             :stream stream
+                             :format-arguments
+                             (list (copy-token-buf-string buf) (package-name pkg))
+                             :format-control
+                             (if accessibility
+                                 "The symbol ~S is not external in the ~A package."
+                                 "Symbol ~S not found in the ~A package.")))))
+                (return (%intern (token-buf-string buf)
+                                 (token-buf-fill-ptr buf)
+                                 pkg
+                                 (if (token-buf-only-base-chars buf)
+                                     (%readtable-symbol-preference rt)
+                                     'character)
+                                 nil)))))))))
 
 ;;; For semi-external use: Return 3 values: the token-buf,
 ;;; a flag for whether there was an escape char, and the position of
@@ -1634,8 +1650,8 @@ extended <package-name>::<form-in-package> syntax."
            (inline token-buf-getchar)) ; makes for smaller code
   (let* ((fixnum-max-digits
           (macrolet ((maxdigits ()
-                       (!coerce-to-specialized  (integer-reader-safe-digits)
-                                                '(unsigned-byte 8))))
+                       (sb-xc:coerce (integer-reader-safe-digits)
+                                     '(vector (unsigned-byte 8)))))
             (aref (maxdigits) (- base 2))))
          (base-power
           (macrolet ((base-powers ()
@@ -1680,9 +1696,9 @@ extended <package-name>::<form-in-package> syntax."
          (magnitude (- number-magnitude divisor-magnitude)))
     (if (minusp exponent)
         (max exponent (ceiling (- (+ max-exponent magnitude))
-                               #.(floor (log 10 2))))
+                               #.(cl:floor (cl:log 10 2))))
         (min exponent (floor (- max-exponent magnitude)
-                             #.(floor (log 10 2)))))))
+                             #.(cl:floor (cl:log 10 2)))))))
 
 (defun make-float (stream)
   ;; Assume that the contents of *read-buffer* are a legal float, with nothing
@@ -1730,7 +1746,8 @@ extended <package-name>::<form-in-package> syntax."
                                   (#\S 'short-float)
                                   (#\F 'single-float)
                                   (#\D 'double-float)
-                                  (#\L 'long-float)))
+                                  (#\L 'long-float)
+                                  (#\R 'rational)))
                   (exponent (truncate-exponent exponent number divisor))
                   (result (make-float-aux (* (expt 10 exponent) number)
                                           divisor float-format stream)))
@@ -1809,23 +1826,31 @@ extended <package-name>::<form-in-package> syntax."
                  (return (setq sub-char (char-upcase ch))))))))
     ;; Look up the function and call it.
     (let ((fn (get-raw-cmt-dispatch-entry sub-char dispatch-table)))
-      (funcall (!cmt-entry-to-function fn #'dispatch-char-error)
+      (funcall (cmt-entry-to-function fn #'dispatch-char-error)
                stream sub-char (if numargp numarg nil)))))
 
 ;;;; READ-FROM-STRING
 
 (declaim (ftype (sfunction (string t t index (or null index) t) (values t index))
-                %read-from-string))
+                %read-from-string
+                %read-from-string/safe))
 (defun %read-from-string (string eof-error-p eof-value start end preserve-whitespace)
-  (with-array-data ((string string :offset-var offset)
-                    (start start)
-                    (end end)
-                    :check-fill-pointer t)
-    (let ((stream (make-string-input-stream string start end)))
-      (values (if preserve-whitespace
-                  (%read-preserving-whitespace stream eof-error-p eof-value nil)
-                  (read stream eof-error-p eof-value))
-              (- (string-input-stream-current stream) offset)))))
+  (let* ((index)
+         (thing
+          (with-input-from-string (stream string :start start :end end :index index)
+            (if preserve-whitespace
+                (%read-preserving-whitespace stream eof-error-p eof-value nil)
+                (read stream eof-error-p eof-value)))))
+    (values thing index)))
+(defun %read-from-string/safe (string eof-error-p eof-value start end preserve-whitespace)
+  (declare (optimize safety)) ; make a stream with indefinite extent
+  (let* ((index)
+         (thing
+          (with-input-from-string (stream string :start start :end end :index index)
+            (if preserve-whitespace
+                (%read-preserving-whitespace stream eof-error-p eof-value nil)
+                (read stream eof-error-p eof-value)))))
+    (values thing index)))
 
 (locally
 (declare (muffle-conditions style-warning))
@@ -1836,7 +1861,7 @@ extended <package-name>::<form-in-package> syntax."
    will take effect."
   (declare (string string))
   (maybe-note-read-from-string-signature-issue eof-error-p)
-  (%read-from-string string eof-error-p eof-value start end preserve-whitespace)))
+  (%read-from-string/safe string eof-error-p eof-value start end preserve-whitespace)))
 
 ;;;; PARSE-INTEGER
 
@@ -1920,7 +1945,7 @@ extended <package-name>::<form-in-package> syntax."
                        and ch from 0
                        when fn do (push (cons (code-char ch) fn) output))
                  (dolist (cell output) ; coerce values to function-designator
-                   (rplacd cell (!cmt-entry-to-fun-designator (cdr cell))))
+                   (rplacd cell (cmt-entry-to-fun-designator (cdr cell))))
                  (when hash-table-p ; caller wants hash-tables
                    (setq output (%stuff-hash-table (make-hash-table) output)))
                  (push (cons char output) alist)))))
