@@ -94,7 +94,7 @@
   (def %instance-set (instance index new-value))
   ;; funcallable instances
   (def %make-funcallable-instance)
-  (def %funcallable-instance-layout)
+  (def %fun-layout)
   (def %set-funcallable-instance-layout (fin new-value))
   (def %funcallable-instance-fun)
   (def (setf %funcallable-instance-fun) (fin new-value))
@@ -134,6 +134,7 @@
   (def stack-ref (s n))
   (def %set-stack-ref (s n value))
   (def fun-code-header)
+  (def symbol-hash)
   (def sb-vm::symbol-extra)
   #+sb-thread (def symbol-tls-index)
   #.(if (fboundp 'symbol-info-vector) (values) '(def symbol-info-vector))
