@@ -173,11 +173,20 @@
 ;;; here, the values are read by an automatically generated reader method.
 (defmethod add-direct-subclass ((class class) (subclass class))
   (with-slots (direct-subclasses) class
-    (pushnew subclass direct-subclasses :test #'eq)
+    (with-world-lock ()
+      (pushnew subclass direct-subclasses :test #'eq))
     subclass))
 (defmethod remove-direct-subclass ((class class) (subclass class))
   (with-slots (direct-subclasses) class
-    (setq direct-subclasses (remove subclass direct-subclasses))
+    (with-world-lock ()
+      (setq direct-subclasses (remove subclass direct-subclasses))
+      ;; Remove from classoid subclasses as well.
+      (let ((classoid (class-classoid subclass)))
+        (dovector (super-layout (layout-inherits (classoid-layout classoid)))
+          (let* ((super (layout-classoid super-layout))
+                 (subclasses (classoid-subclasses super)))
+            (when subclasses
+              (remhash classoid subclasses))))))
     subclass))
 
 ;;; Maintaining the direct-methods and direct-generic-functions backpointers.
@@ -198,7 +207,7 @@
 ;;; This needs to be used recursively, in case a non-trivial user
 ;;; defined ADD/REMOVE-DIRECT-METHOD method ends up calling another
 ;;; function using the same lock.
-(defvar *specializer-lock* (sb-thread:make-mutex :name "Specializer lock"))
+(define-load-time-global *specializer-lock* (sb-thread:make-mutex :name "Specializer lock"))
 
 (defmethod add-direct-method :around ((specializer specializer) method)
   ;; All the actions done under this lock are done in an order
@@ -280,7 +289,8 @@
 ;;; This table is shared between threads, so needs to be synchronized.
 ;;; (though insertions are only performed when holding the specializer-lock,
 ;;; so the preceding claim is probably overly paranoid.)
-(defvar *class-eq-specializer-methods* (make-hash-table :test 'eq :synchronized t))
+(define-load-time-global *class-eq-specializer-methods*
+    (make-hash-table :test 'eq :synchronized t))
 
 (defmethod specializer-method-table ((specializer class-eq-specializer))
   *class-eq-specializer-methods*)
@@ -290,7 +300,7 @@
   (let ((table (specializer-method-table self))
         (object (specializer-object self)))
     (if create
-        (sb-impl::puthash-if-absent object table (lambda () (cons nil nil)))
+        (ensure-gethash object table (cons nil nil) t)
         (gethash object table))))
 
 (defun map-specializers (function)
@@ -333,7 +343,7 @@
      (apply #'ensure-class name :metaclass metaclass-name
             :direct-superclasses supers
             :direct-slots slots
-            :definition-source source-location
+            'source source-location
             'safe-p safe-p
             other))))
 
@@ -366,7 +376,7 @@
                   (cond
                     ((eq class-or-name name)
                      (error "~@<Class named ~
-                             ~/sb-impl::print-symbol-with-prefix/ ~
+                             ~/sb-ext:print-symbol-with-prefix/ ~
                              specified as its own ~(~A~)class.~@:>"
                             class-or-name which))
                     ((find-class class-or-name (eq which :meta)))
@@ -449,11 +459,8 @@
                   (mapcar (lambda (pl) (make-direct-slotd class pl))
                           direct-slots))
             (slot-value class 'direct-slots)))
-  (if direct-default-initargs-p
-      (setf (plist-value class 'direct-default-initargs)
-            direct-default-initargs)
-      (setq direct-default-initargs
-            (plist-value class 'direct-default-initargs)))
+  (when direct-default-initargs-p
+    (setf (plist-value class 'direct-default-initargs) direct-default-initargs))
   (setf (plist-value class 'class-slot-cells)
         (let ((old-class-slot-cells (plist-value class 'class-slot-cells))
               (safe (safe-p class))
@@ -500,10 +507,10 @@
 
 (defmethod invalid-superclass ((class class) (superclass class))
   (error 'invalid-superclass :class class :superclass superclass
-         :references (list* '(:amop :generic-function validate-superclass)
-                            (and (typep superclass 'built-in-class)
-                                 (list '(:ansi-cl :system-class built-in-class)
-                                       '(:ansi-cl :section (4 3 7)))))))
+         :references `((:amop :generic-function validate-superclass)
+                       ,@(when (typep superclass 'built-in-class)
+                           '((:ansi-cl :system-class built-in-class)
+                             (:ansi-cl :section (4 3 7)))))))
 
 (defmethod shared-initialize :after ((class forward-referenced-class)
                                      slot-names &key &allow-other-keys)
@@ -531,10 +538,10 @@
             (let ((layout (make-wrapper 0 class)))
               (setf (slot-value class 'wrapper) layout)
               (let ((cpl (compute-preliminary-cpl class)))
-                (setf (layout-inherits layout)
-                      (order-layout-inherits
-                       (map 'simple-vector #'class-wrapper
-                            (reverse (rest cpl))))))
+                (set-layout-inherits layout
+                                     (order-layout-inherits
+                                      (map 'simple-vector #'class-wrapper
+                                           (reverse (rest cpl))))))
               (register-layout layout :invalidate t))))
         (mapc #'make-preliminary-layout (class-direct-subclasses class))))))
 
@@ -563,7 +570,7 @@
   (let* ((name (class-name class))
          (classoid (find-classoid name))
          (slots (condition-classoid-slots classoid))
-         (source (sb-kernel::layout-source-location (classoid-layout classoid))))
+         (source (sb-kernel::classoid-source-location classoid)))
     ;; to balance the REMOVE-SLOT-ACCESSORS call in
     ;; REINITIALIZE-INSTANCE :BEFORE (SLOT-CLASS).
     (flet ((add-source-location (method)
@@ -921,7 +928,7 @@
 (define-condition cpl-protocol-violation (reference-condition error)
   ((class :initarg :class :reader cpl-protocol-violation-class)
    (cpl :initarg :cpl :reader cpl-protocol-violation-cpl))
-  (:default-initargs :references (list '(:sbcl :node "Metaobject Protocol")))
+  (:default-initargs :references '((:sbcl :node "Metaobject Protocol")))
   (:report
    (lambda (c s)
      (format s "~@<Protocol violation: the ~S class ~S ~
@@ -1030,7 +1037,7 @@
           (style-warn
            "~@<slot names with the same SYMBOL-NAME but ~
                   different SYMBOL-PACKAGE (possible package problem) ~
-                  for class ~S:~4I~@:_~<~@{~/sb-impl::print-symbol-with-prefix/~^~:@_~}~:>~@:>"
+                  for class ~S:~4I~@:_~<~@{~/sb-ext:print-symbol-with-prefix/~^~:@_~}~:>~@:>"
            class dupes)))
     (let* ((slot-name (slot-definition-name (car slots)))
            (oslots (and (not (eq (symbol-package slot-name)
@@ -1196,6 +1203,9 @@
                      cell)))))
       (unless (slot-definition-class eslotd)
         (setf (slot-definition-class eslotd) class))
+      (let ((info (slot-definition-info eslotd)))
+        (setf (slot-info-allocation info) (slot-definition-allocation eslotd)
+              (slot-info-location info) (slot-definition-location eslotd)))
       (initialize-internal-slot-functions eslotd))))
 
 (defmethod compute-slots :around ((class standard-class))
@@ -1225,18 +1235,18 @@
 
 (defmethod compute-effective-slot-definition-initargs
     ((class slot-class) direct-slotds)
-  (let* ((name nil)
-         (initfunction nil)
-         (initform nil)
-         (initargs nil)
-         (allocation nil)
-         (allocation-class nil)
-         (type t)
-         (documentation nil)
-         (documentationp nil)
-         (namep  nil)
-         (initp  nil)
-         (allocp nil))
+  (let ((name nil)
+        (initfunction nil)
+        (initform nil)
+        (initargs nil)
+        (allocation nil)
+        (allocation-class nil)
+        (type t)
+        (documentation nil)
+        (documentationp nil)
+        (namep  nil)
+        (initp  nil)
+        (allocp nil))
 
     (dolist (slotd direct-slotds)
       (when slotd
@@ -1256,7 +1266,7 @@
           (setq allocation (slot-definition-allocation slotd)
                 allocation-class (slot-definition-class slotd)
                 allocp t))
-        (setq initargs (append (slot-definition-initargs slotd) initargs))
+        (setq initargs (union (slot-definition-initargs slotd) initargs))
         (let ((slotd-type (slot-definition-type slotd)))
           (setq type (cond
                        ((eq type t) slotd-type)
@@ -1305,7 +1315,7 @@
                              :slot-name slot-name
                              :object-class class
                              :method-class-function #'reader-method-class
-                             :definition-source source-location)))
+                             'source source-location)))
 
 (defmethod writer-method-class ((class slot-class) direct-slot &rest initargs)
   (declare (ignore direct-slot initargs))
@@ -1322,7 +1332,7 @@
                              :slot-name slot-name
                              :object-class class
                              :method-class-function #'writer-method-class
-                             :definition-source source-location)))
+                             'source source-location)))
 
 (defmethod add-boundp-method ((class slot-class) generic-function slot-name slot-documentation source-location)
   (add-method generic-function
@@ -1334,19 +1344,24 @@
                              (make-boundp-method-function class slot-name)
                              (or slot-documentation "automatically generated boundp method")
                              :slot-name slot-name
-                             :definition-source source-location)))
+                             'source source-location)))
 
 (defmethod remove-reader-method ((class slot-class) generic-function)
-  (let ((method (get-method generic-function () (list class) nil)))
+  (let ((method
+         (and (= (length (arg-info-metatypes (gf-arg-info generic-function))) 1)
+              (get-method generic-function () (list class) nil))))
     (when method (remove-method generic-function method))))
 
 (defmethod remove-writer-method ((class slot-class) generic-function)
   (let ((method
-          (get-method generic-function () (list *the-class-t* class) nil)))
+         (and (= (length (arg-info-metatypes (gf-arg-info generic-function))) 2)
+              (get-method generic-function () (list *the-class-t* class) nil))))
     (when method (remove-method generic-function method))))
 
 (defmethod remove-boundp-method ((class slot-class) generic-function)
-  (let ((method (get-method generic-function () (list class) nil)))
+  (let ((method
+         (and (= (length (arg-info-metatypes (gf-arg-info generic-function))) 1)
+              (get-method generic-function () (list class) nil))))
     (when method (remove-method generic-function method))))
 
 ;;; MAKE-READER-METHOD-FUNCTION and MAKE-WRITER-METHOD-FUNCTION
@@ -1486,18 +1501,23 @@
                                 old-class new-class)
   (do () ((typep value slot-type))
     (restart-case
-        (bad-type value slot-type
+        (error 'simple-type-error
+               :datum value
+               :expected-type slot-type
+               :format-control
+                  (sb-format:tokens
                   "~@<Error during ~A. Current value in slot ~
-                   ~/sb-impl::print-symbol-with-prefix/ of an instance ~
+                   ~/sb-ext:print-symbol-with-prefix/ of an instance ~
                    of ~S is ~S, which does not match the new slot type ~
-                   ~S in class ~S.~:@>"
-                  context slot-name old-class value slot-type new-class)
+                   ~S in class ~S.~:@>")
+               :format-arguments
+               (list context slot-name old-class value slot-type new-class))
       (use-value (new-value)
         :interactive read-evaluated-form
         :report (lambda (stream)
                   (format stream "~@<Specify a new value to by used ~
                                   for slot ~
-                                  ~/sb-impl::print-symbol-with-prefix/ ~
+                                  ~/sb-ext:print-symbol-with-prefix/ ~
                                   instead of ~S.~@:>"
                           slot-name value))
         (setf value new-value))))
@@ -1505,13 +1525,13 @@
 
 (defun %set-slot-value-checking-type (context slots slot value
                                       safe old-class new-class)
-  (setf (clos-slots-ref slots (slot-definition-location slot))
-        (if (and safe (not (unbound-marker-p value)))
-            (let ((name (slot-definition-name slot))
-                  (type (slot-definition-type slot)))
-              (%ensure-slot-value-type context name type value
-                                       old-class new-class))
-            value)))
+  (let ((value (if (and safe (not (unbound-marker-p value)))
+                   (let ((name (slot-definition-name slot))
+                         (type (slot-definition-type slot)))
+                     (%ensure-slot-value-type context name type value
+                                              old-class new-class))
+                   value)))
+    (setf (clos-slots-ref slots (slot-definition-location slot)) value)))
 
 (defvar *in-obsolete-instance-trap* nil)
 
@@ -1526,11 +1546,12 @@
 
 (defun %obsolete-instance-trap (owrapper nwrapper instance)
   (cond
-    ((layout-for-std-class-p owrapper)
+    ((layout-for-pcl-obj-p owrapper)
      (binding* ((class (wrapper-class* nwrapper))
-                (copy (allocate-instance class)) ;??? allocate-instance ???
                 (oslots (get-slots instance))
-                (nslots (get-slots copy))
+                (nwrapper (class-wrapper class))
+                (nslots (make-array (layout-length nwrapper)
+                                    :initial-element +slot-unbound+))
                 (added ())
                 (discarded ())
                 (plist ())
@@ -1566,7 +1587,16 @@
          ;; Go through all the old local slots.
          (dolist (old old-instance-slots)
            (let* ((name (slot-definition-name old))
-                  (value (clos-slots-ref oslots (slot-definition-location old))))
+                  (cell (find-slot-cell owrapper name))
+                  (location (car cell))
+                  (value (cond
+                           ((fixnump location)
+                            (clos-slots-ref oslots location))
+                           ((not location)
+                            (let ((location (slot-info-location (cdr cell))))
+                              (aver (integerp location))
+                              (clos-slots-ref oslots (slot-info-location (cdr cell)))))
+                           (t (bug "non-FIXNUM non-NULL location in cell: ~S" cell)))))
              (unless (unbound-marker-p value)
                (let ((new (assq name layout)))
                  (cond (new
@@ -1598,7 +1628,11 @@
          (dolist (cell layout)
            (push (car cell) added)))
 
-       (%swap-wrappers-and-slots instance copy)
+       (if (std-instance-p instance)
+           (setf (%instance-layout instance) nwrapper
+                 (std-instance-slots instance) nslots)
+           (setf (%fun-layout instance) nwrapper
+                 (%fsc-instance-slots instance) nslots))
 
        (update-instance-for-redefined-class
         instance added discarded plist)
@@ -1612,29 +1646,28 @@
 
 (defun %change-class (instance new-class initargs)
   (declare (notinline allocate-instance))
-  (binding* ((old-class (class-of instance))
+  (binding* ((old-wrapper (layout-of instance))
+             (old-class (wrapper-class* old-wrapper))
              (copy (allocate-instance new-class))
-             (new-wrapper (get-wrapper copy))
-             (old-wrapper (class-wrapper old-class))
+             (new-wrapper (let ((layout (layout-of copy)))
+                            (aver (layout-for-pcl-obj-p layout))
+                            (aver (= (layout-bitmap old-wrapper)
+                                     (layout-bitmap layout)))
+                            layout))
              (old-slots (get-slots instance))
              (new-slots (get-slots copy))
              (safe (safe-p new-class))
-             (new-wrapper-slots (layout-slot-list new-wrapper))
-             (old-wrapper-slots (layout-slot-list old-wrapper)))
-    (labels ((find-instance-slot (name slots)
-               (loop for slot in slots
-                     when (and (eq (slot-definition-allocation slot) :instance)
-                               (eq (slot-definition-name slot) name))
-                     return slot))
-             (initarg-for-slot-p (slot)
+             (new-wrapper-slots (layout-slot-list new-wrapper)))
+    (flet ((initarg-for-slot-p (slot)
+             (when initargs
                (dolist (slot-initarg (slot-definition-initargs slot))
                  (unless (unbound-marker-p
                           (getf initargs slot-initarg +slot-unbound+))
-                   (return t))))
-             (set-value (value slotd)
-               (%set-slot-value-checking-type
-                'change-class new-slots slotd value safe
-                old-class new-class)))
+                   (return t)))))
+           (set-value (value slotd)
+             (%set-slot-value-checking-type
+              'change-class new-slots slotd value safe
+              old-class new-class)))
 
       ;; "The values of local slots specified by both the class CTO
       ;; and CFROM are retained. If such a local slot was unbound, it
@@ -1642,25 +1675,42 @@
       (dolist (new new-wrapper-slots)
         (when (and (not (initarg-for-slot-p new))
                    (eq (slot-definition-allocation new) :instance))
-          (binding* ((old (find-instance-slot (slot-definition-name new) old-wrapper-slots)
-                          :exit-if-null)
-                     (value (clos-slots-ref old-slots (slot-definition-location old))))
-            (set-value value new))))
-
-      ;; "The values of slots specified as shared in the class CFROM and
-      ;; as local in the class CTO are retained."
-      (dolist (old old-wrapper-slots)
-        (when (eq (slot-definition-allocation old) :class)
-         (binding* ((slot-and-val (slot-definition-location old))
-                    (new (find-instance-slot (car slot-and-val) new-wrapper-slots)
-                         :exit-if-null))
-           (set-value (cdr slot-and-val) new)))))
+          (binding* ((cell (find-slot-cell old-wrapper (slot-definition-name new))
+                           :exit-if-null)
+                     (location (car cell))
+                     (value (cond
+                              ((fixnump location)
+                               (clos-slots-ref old-slots location))
+                              ((not location)
+                               (let ((info (cdr cell)))
+                                 (case (slot-info-allocation info)
+                                   (:instance
+                                    (clos-slots-ref old-slots (slot-info-location info)))
+                                   (:class (cdr (slot-info-location info))))))
+                              (t
+                               (cdr location)))))
+            (set-value value new)))))
 
     ;; Make the copy point to the old instance's storage, and make the
     ;; old instance point to the new storage.
-    (%swap-wrappers-and-slots instance copy)
+    ;; All uses of %CHANGE-CLASS are under the world lock, but that doesn't
+    ;; preclude user code operating on the old slots + new layout or v.v.
+    ;; Users need to synchronize their own access when changing class.
+    (cond ((std-instance-p instance)
+           (rotatef (%instance-layout instance) (%instance-layout copy))
+           (rotatef (%std-instance-slots instance) (%std-instance-slots copy)))
+          (t
+           (rotatef (%fun-layout instance) (%fun-layout copy))
+           (rotatef (%fsc-instance-slots instance) (%fsc-instance-slots copy))))
 
     (apply #'update-instance-for-different-class copy instance initargs)
+    ;; If the user subsequently operates on COPY, crash and burn. As per CLHS:
+    ;;  "The first argument to update-instance-for-different-class, /previous/,
+    ;;  is that copy; it holds the old slot values temporarily. This argument has
+    ;;  dynamic extent within change-class; if it is referenced in any way once
+    ;;  update-instance-for-different-class returns, the results are undefined."
+    (cond ((std-instance-p copy) (setf (%std-instance-slots copy) 0))
+          ((fsc-instance-p copy) (setf (%fsc-instance-slots copy) 0)))
 
     instance))
 
@@ -1747,10 +1797,9 @@
              `(defmethod ,name ,args
                 (declare (ignore initargs))
                 (error 'metaobject-initialization-violation
-                       :format-control ,(coerce (format nil "~@<~A~@:>" control)
-                                                'base-string)
+                       :format-control ,(format nil "~~@<~A~~@:>" control)
                        :format-arguments (list (class-name class))
-                       :references (list '(:amop :initialization "Class"))))))
+                       :references '((:amop :initialization "Class"))))))
   (def initialize-instance ((class system-class) &rest initargs)
     "Cannot initialize an instance of ~S.")
   (def reinitialize-instance ((class system-class) &rest initargs)
