@@ -81,6 +81,23 @@ static inline int code_text_size(struct code* c) {
 static inline lispobj* code_jumptable_start(struct code* code) {
     return code->boxed_size ? (lispobj*)code_text_start(code) : 0;
 }
+static inline unsigned int jumptable_count(lispobj* table) {
+    // extract low 14 bits regardless of machine word size
+    return table ? *table & 0x3FFF : 0;
+}
+static inline unsigned int code_serialno(struct code* code) {
+    // extract next 18 bits regardless of machine word size
+    lispobj* table = code_jumptable_start(code);
+    return table ? *table >> 14 : 0;
+}
+
+static inline unsigned int code_n_named_calls(struct code* code) {
+#ifdef LISP_FEATURE_64_BIT
+    return code->boxed_size >> 32;
+#else
+    return 0;
+#endif
+}
 
 // How many elements in 'code->constants[]' are taken by each simple-fun
 #define CODE_SLOTS_PER_SIMPLE_FUN 4
@@ -135,6 +152,21 @@ static inline lispobj fun_code_tagged(lispobj* fun) {
     return make_lispobj(fun_code_header(fun), OTHER_POINTER_LOWTAG);
 }
 
+#ifdef RETURN_PC_WIDETAG
+#define embedded_obj_p(tag) (tag==RETURN_PC_WIDETAG || tag==SIMPLE_FUN_WIDETAG)
+#else
+#define embedded_obj_p(tag) (tag==SIMPLE_FUN_WIDETAG)
+#endif
+/* Convert from a lispobj with lowtag bits to the starting address
+ * of the heap object. */
+static inline lispobj *
+base_pointer(lispobj ptr)
+{
+    lispobj *obj = native_pointer(ptr);
+    int widetag = widetag_of(obj);
+    return embedded_obj_p(widetag) ? fun_code_header(obj) : obj;
+}
+
 #if defined LISP_FEATURE_X86 || defined LISP_FEATURE_X86_64
 # define fun_self_from_baseptr(simple_fun) (lispobj)simple_fun->insts
 # define fun_self_from_taggedptr(funptr) \
