@@ -19,8 +19,6 @@
 #include "immobile-space.h"
 #include "code.h"
 
-#define CUSTOM_GC_SCAVENGE_FLAG 0x800000
-
 #ifdef LISP_FEATURE_GENCGC
 #include "gencgc-alloc-region.h"
 static inline void *
@@ -50,12 +48,10 @@ extern void *gc_general_alloc(sword_t nbytes,int page_type_flag);
 static inline lispobj
 gc_general_copy_object(lispobj object, long nwords, int page_type_flag)
 {
-    lispobj *new;
-
     CHECK_COPY_PRECONDITIONS(object, nwords);
 
     /* Allocate space. */
-    new = gc_general_alloc(nwords*N_WORD_BYTES, page_type_flag);
+    lispobj *new = gc_general_alloc(nwords*N_WORD_BYTES, page_type_flag);
 
     /* Copy the object. */
     memcpy(new,native_pointer(object),nwords*N_WORD_BYTES);
@@ -65,6 +61,19 @@ gc_general_copy_object(lispobj object, long nwords, int page_type_flag)
     return make_lispobj(new, lowtag_of(object));
 }
 
+// Like above but copy potentially fewer words than are allocated.
+// ('old_nwords' can be, but does not have to be, smaller than 'nwords')
+static inline lispobj
+gc_copy_object_resizing(lispobj object, long nwords, int page_type_flag,
+                        int old_nwords)
+{
+    CHECK_COPY_PRECONDITIONS(object, nwords);
+    lispobj *new = gc_general_alloc(nwords*N_WORD_BYTES, page_type_flag);
+    memcpy(new, native_pointer(object), old_nwords*N_WORD_BYTES);
+    note_transported_object(object, new);
+    return make_lispobj(new, lowtag_of(object));
+}
+
 extern sword_t (*const scavtab[256])(lispobj *where, lispobj object);
 extern struct cons *weak_vectors; /* in gc-common.c */
 extern struct hash_table *weak_hash_tables; /* in gc-common.c */
@@ -331,6 +340,8 @@ static inline lispobj layout_of(lispobj* instance) { // native ptr
     return widetag_of(instance) == FUNCALLABLE_INSTANCE_WIDETAG
       ? funinstance_layout(instance) : instance_layout(instance);
 }
+
+extern lispobj layout_of_layout;
 /// Return true if 'thing' is a layout.
 static inline boolean layoutp(lispobj thing)
 {
@@ -338,7 +349,7 @@ static inline boolean layoutp(lispobj thing)
     lispobj layout;
     if ((base_ptr & LOWTAG_MASK) || !(layout = layout_of((lispobj*)base_ptr)))
         return 0;
-    return LAYOUT(layout)->flags & flag_LayoutLayout;
+    return layout == layout_of_layout;
 }
 
 static inline int lockfree_list_node_layout_p(struct layout* layout) {
