@@ -23,8 +23,6 @@
 #endif
 #include "runtime.h"
 
-#if defined(LISP_FEATURE_SB_LDB)
-
 #include "globals.h"
 #include "vars.h"
 #include "parse.h"
@@ -36,7 +34,7 @@
 #include "arch.h"
 #include "search.h"
 #include "thread.h"
-#include "pseudo-atomic.h"
+#include "getallocptr.h"
 
 #include "genesis/simple-fun.h"
 #include "genesis/fdefn.h"
@@ -153,94 +151,90 @@ char *parse_token(char **ptr)
     return token;
 }
 
-uword_t parse_number(char **ptr)
+/// Return 1 for successful parse
+int parse_number(char **ptr, int *output)
 {
     char *token = parse_token(ptr);
     uword_t result;
 
     if (token == NULL) {
         printf("expected a number\n");
-        throw_to_monitor();
+        return 0;
     }
-    else if (string_to_long(token, &result))
-        return result;
-    else {
-        printf("invalid number: ``%s''\n", token);
-        throw_to_monitor();
+    if (string_to_long(token, &result)) {
+        // FIXME: consumers of this interface expect to receive an 'int',
+        // but string_to_long yields a uword_t.
+        // Should enforce the smaller range.
+        *output = result;
+        return 1;
     }
+    printf("invalid number: ``%s''\n", token);
     return 0;
 }
 
-char *parse_addr(char **ptr, boolean safely)
+int parse_addr(char **ptr, boolean safely, char **output)
 {
     char *token = parse_token(ptr);
     lispobj result;
 
     if (token == NULL) {
         printf("expected an address\n");
-        throw_to_monitor();
+        return 0;
     }
-    else if (token[0] == '$') {
+    if (token[0] == '$') {
         if (!lookup_variable(token+1, &result)) {
             printf("unknown variable: ``%s''\n", token);
-            throw_to_monitor();
+            return 0;
         }
-        result &= ~7;
-    }
-    else {
+        result &= ~7; // LOWTAG_MASK maybe?
+    } else {
         uword_t value;
         if (!string_to_long(token, &value)) {
             printf("invalid number: ``%s''\n", token);
-            throw_to_monitor();
+            return 0;
         }
-        result = (value & ~3);
+        result = (value & ~3); // what is ~3 for - word alignment?
     }
 
-    if (safely && !is_valid_lisp_addr((os_vm_address_t)result)) {
+    if (safely && !gc_managed_addr_p(result)) {
         printf("invalid Lisp-level address: %p\n", (void *)result);
-        throw_to_monitor();
+        return 0;
     }
 
-    return (char *)result;
+    *output = (char *)result;
+    return 1;
 }
 
 static lispobj lookup_symbol(char *name)
 {
-    int count;
-    lispobj *headerptr;
-
-    /* Search static space. */
-    headerptr = (lispobj *)STATIC_SPACE_START;
-    count =
-        (lispobj *)SymbolValue(STATIC_SPACE_FREE_POINTER,0) -
-        (lispobj *)STATIC_SPACE_START;
-    if (search_for_symbol(name, &headerptr, &count))
-        return make_lispobj(headerptr, OTHER_POINTER_LOWTAG);
-
+    uword_t ranges[][2] = {
+      { STATIC_SPACE_START, (uword_t)static_space_free_pointer },
 #ifdef LISP_FEATURE_IMMOBILE_SPACE
-    /* Search immobile space. */
-    headerptr = (lispobj *)IMMOBILE_SPACE_START;
-    count = IMMOBILE_FIXEDOBJ_SUBSPACE_SIZE / N_WORD_BYTES;
-    if (search_for_symbol(name, &headerptr, &count))
-        return make_lispobj(headerptr, OTHER_POINTER_LOWTAG);
+      { FIXEDOBJ_SPACE_START, (uword_t)fixedobj_free_pointer },
 #endif
-
-    /* Search dynamic space. */
 #if defined(LISP_FEATURE_GENCGC)
-    headerptr = (lispobj *)DYNAMIC_SPACE_START;
-    count = (lispobj *)get_alloc_pointer() - headerptr;
+      { DYNAMIC_SPACE_START, (uword_t)get_alloc_pointer() }
 #else
-    headerptr = (lispobj *)current_dynamic_space;
-    count = dynamic_space_free_pointer - headerptr;
+      { (uword_t)current_dynamic_space, (uword_t)get_alloc_pointer() }
 #endif
-    if (search_for_symbol(name, &headerptr, &count))
-        return make_lispobj(headerptr, OTHER_POINTER_LOWTAG);
+    };
 
+    lispobj *headerptr;
+    unsigned i;
+    for (i=0; i<(sizeof ranges/(2*sizeof (uword_t))); ++i)
+        if ((headerptr = search_for_symbol(name, ranges[i][0], ranges[i][1], 0)))
+            return make_lispobj(headerptr, OTHER_POINTER_LOWTAG);
+    // Try again, case-insensitively
+    for (i=0; i<(sizeof ranges/(2*sizeof (uword_t))); ++i)
+        if ((headerptr = search_for_symbol(name, ranges[i][0], ranges[i][1], 1)))
+            return make_lispobj(headerptr, OTHER_POINTER_LOWTAG);
     return 0;
 }
 
 static int parse_regnum(char *s)
 {
+    // Well this is confusing as heck! Registers r0..r9 in the AARCH64 vm definition
+    // are machine regs 10 through 19. So caveat emptor if you use this syntax.
     if ((s[1] == 'R') || (s[1] == 'r')) {
         int regnum;
 
@@ -268,7 +262,7 @@ static int parse_regnum(char *s)
     }
 }
 
-lispobj parse_lispobj(char **ptr)
+int parse_lispobj(char **ptr, lispobj *output)
 {
     struct thread *thread=arch_os_get_current_thread();
     char *token = parse_token(ptr);
@@ -278,46 +272,50 @@ lispobj parse_lispobj(char **ptr)
 
     if (token == NULL) {
         printf("expected an object\n");
-        throw_to_monitor();
-    } else if (token[0] == '$') {
+        return 0;
+    }
+    if (token[0] == '$') {
         if (isalpha(token[1])) {
             int free_ici;
             int regnum;
             os_context_t *context;
 
-            free_ici = fixnum_value(SymbolValue(FREE_INTERRUPT_CONTEXT_INDEX,thread));
+            free_ici = fixnum_value(read_TLS(FREE_INTERRUPT_CONTEXT_INDEX,thread));
 
             if (free_ici == 0) {
                 printf("Variable ``%s'' is not valid -- there is no current interrupt context.\n", token);
-                throw_to_monitor();
+                return 0;
             }
 
-            context = thread->interrupt_contexts[free_ici - 1];
+            context = nth_interrupt_context(free_ici - 1, thread);
 
             regnum = parse_regnum(token);
             if (regnum < 0) {
                 printf("bogus register: ``%s''\n", token);
-                throw_to_monitor();
+                return 0;
             }
 
             result = *os_context_register_addr(context, regnum);
         } else if (!lookup_variable(token+1, &result)) {
             printf("unknown variable: ``%s''\n", token);
-            throw_to_monitor();
+            return 0;
         }
     } else if (token[0] == '@') {
         if (string_to_long(token+1, &pointer)) {
+            // More wtf - are we trying to round down to a double-lispword
+            // or to a lispword? I don't know.
+            // This '@' input syntax is just plain useless anyway.
             pointer &= ~3;
-            if (is_valid_lisp_addr((os_vm_address_t)pointer))
+            if (gc_managed_addr_p(pointer))
                 result = *(lispobj *)pointer;
             else {
                 printf("invalid Lisp-level address: ``%s''\n", token+1);
-                throw_to_monitor();
+                return 0;
             }
         }
         else {
             printf("invalid address: ``%s''\n", token+1);
-            throw_to_monitor();
+            return 0;
         }
     }
     else if (string_to_long(token, &value))
@@ -326,10 +324,9 @@ lispobj parse_lispobj(char **ptr)
         ;
     else {
         printf("invalid Lisp object: ``%s''\n", token);
-        throw_to_monitor();
+        return 0;
     }
 
-    return result;
+    *output = result;
+    return 1;
 }
-
-#endif /* defined(LISP_FEATURE_SB_LDB) */
