@@ -147,7 +147,7 @@
 
 (with-test (:name (sb-vm:map-allocated-objects :no-consing)
             :serial t
-            :fails-on :cheneygc
+            :fails-on (or :cheneygc (not :sb-thread))
             :skipped-on :interpreter)
   (let ((n 0))
     (sb-int:dx-flet ((f (obj type size)
@@ -222,10 +222,10 @@
         (incf count)))
     (assert (= count 2))))
 
-(with-test (:name :linkage-table-bogosity :skipped-on (not :sb-dynamic-core))
+;; Whoever deals with the Alpha and/or HPPA ports can make this test pass for them
+(with-test (:name :linkage-table-bogosity)
   (let ((strings (map 'list (lambda (x) (if (consp x) (car x) x))
-                      #+sb-dynamic-core sb-vm::+required-foreign-symbols+
-                      #-sb-dynamic-core '())))
+                      sb-vm::+required-foreign-symbols+)))
     (assert (= (length (remove-duplicates strings :test 'string=))
                (length strings)))))
 
@@ -2475,17 +2475,27 @@
                (rem x 10)))))
         (assert (not (ctu:find-code-constants f :type 'bignum)))))
 
-(with-test (:name :deduplicated-fdefns)
-  (dolist (c (sb-vm::list-allocated-objects :all :type sb-vm:code-header-widetag))
-    (let (dup-fdefns names)
-      (dotimes (i (sb-kernel:code-header-words c))
-        (let ((obj (sb-kernel:code-header-ref c i)))
-          (when (sb-kernel:fdefn-p obj)
-            (let ((name (sb-kernel:fdefn-name obj)))
-              (when (member name names)
-                (push obj dup-fdefns))
-              (push name names)))))
-      (assert (not dup-fdefns)))))
+(with-test (:name :deduplicated-fdefns :fails-on (not :64-bit))
+  (flet ((scan-range (c start end)
+           (let (dup-fdefns names)
+             (loop for i from start below end
+                   do (let ((obj (sb-kernel:code-header-ref c i)))
+                        (when (sb-kernel:fdefn-p obj)
+                          (let ((name (sb-kernel:fdefn-name obj)))
+                            (when (member name names)
+                              (push obj dup-fdefns))
+                            (push name names)))))
+             (assert (not dup-fdefns)))))
+    (dolist (c (sb-vm::list-allocated-objects :all :type sb-vm:code-header-widetag))
+      (let* ((start (+ sb-vm:code-constants-offset
+                       (* (sb-kernel:code-n-entries c)
+                          sb-vm:code-slots-per-simple-fun)))
+             (end (+ start (sb-kernel:code-n-named-calls c))))
+        ;; Within each subset of FDEFNs there should be no duplicates
+        ;; by name. But there could be an fdefn that is in the union of
+        ;; the ranges twice, if used for named call and a global ref.
+        (scan-range c start end)
+        (scan-range c end (sb-kernel:code-header-words c))))))
 
 (with-test (:name :map-all-lvar-dests)
   (checked-compile-and-assert
@@ -2748,7 +2758,7 @@
                         (cond ((>= sum '0)
                                (go end))
                               (a
-                               (ceiling 1 (unwind-protect 0))
+                               (ceiling 1 (unwind-protect 2))
                                (incf sum)))
                         (go next)
                       end))
@@ -2792,3 +2802,97 @@
                    (funcall z))))
     ((nil) 123)
     ((t) 123)))
+
+(with-test (:name :values-list+cons)
+  (assert
+   (equal (sb-kernel:%simple-fun-type
+           (checked-compile
+            `(lambda ()
+               (values-list (cons 1 nil)))))
+          '(function () (values (integer 1 1) &optional)))))
+
+(with-test (:name :xeps-and-inlining)
+  (checked-compile-and-assert
+   ()
+   `(lambda (args)
+      (flet ((fun () args))
+        (declare (inline fun))
+        (multiple-value-call #'fun (values-list args))
+        #'fun))))
+
+(with-test (:name :split-let-ctran-kind)
+  (checked-compile-and-assert
+   ()
+   `(lambda (a b)
+      (let ((a-n (null a))
+            (b-n (null b)))
+        (cond (b-n 1)
+              (a-n a)
+              (t a))))
+   ((nil nil) 1)
+   ((nil t) nil)))
+
+(with-test (:name :dead-component-unused-closure)
+  (checked-compile-and-assert
+   ()
+   `(lambda ()
+      (labels ((%f1 ())
+               (%f2 (&key)
+                 (flet ((%f3 ()
+                          (unwind-protect 1)
+                          (return-from %f2 (%f1)))))))
+        (%f1)))
+   (() nil)))
+
+(with-test (:name :references-to-inline-funs-copied)
+  (checked-compile-and-assert
+      ()
+      `(lambda ()
+         (and (inline-fun-arg-mismatch t)
+              #'inline-fun-arg-mismatch))
+    (() #'inline-fun-arg-mismatch)))
+
+(with-test (:name :eliminate-dead-code-before-initial-dfo)
+  (checked-compile-and-assert
+      ()
+      `(lambda ()
+         (block nil
+          (flet ((f (&key (k1 (catch 'c)))
+                   (max 0
+                        (let ((v9 10))
+                          (return))))))))
+    (() nil)))
+
+(with-test (:name :%coerce-callable-to-fun-movement)
+  (checked-compile-and-assert
+   ()
+   `(lambda (y x)
+      (let ((x (sb-kernel:%coerce-callable-to-fun x)))
+        (when y
+          (funcall x))))
+    ((nil (make-symbol "UNDEF")) (condition 'undefined-function))))
+
+(with-test (:name :jump-table-use-labels)
+  (checked-compile-and-assert
+   ()
+   `(lambda (x m)
+      (case x
+        ((a b c)
+         (if m
+             (error ""))
+         x)
+        ((d e f)
+         (eval 10)
+         x)))
+    (('a nil) 'a)
+    (('d 30) 'd)))
+
+(with-test (:name :dfo-deleted-lambda-home)
+  (assert
+   (nth-value 5 (checked-compile
+                 `(lambda (c)
+                    (flet ((f (&optional (o c))
+                             (lambda (&key)
+                               (+ (restart-bind nil (go missing-tag))
+                                  (progv nil nil o)))))))
+                 :allow-failure t))))
