@@ -17,7 +17,7 @@
 ;;;; are order-preserving - unless they are of the form (CONS (EQL x)).
 ;;;; This is not a requirement in general, but is quite reasonable.
 (with-test (:name :pprint-dispatch-order-preserving)
-  (let ((tbl (sb-pretty::make-pprint-dispatch-table)))
+  (let ((tbl (sb-pretty::make-pprint-dispatch-table nil nil nil)))
     (handler-bind ((warning #'muffle-warning)) ; nonexistent types
       ;; use EVAL because there are *two* warnings to muffle: first time
       ;; is when the compiler sees a symbol used as an unknown type-specifier,
@@ -450,4 +450,18 @@
   (assert (= (sb-pretty::macro-indentation 'try5) 2))
   (assert (= (sb-pretty::macro-indentation 'try6) 3)))
 
+(defclass ship () ())
+(let ((ppd (copy-pprint-dispatch)))
+  (set-pprint-dispatch 'integer
+                       (lambda (stream obj)
+                         (let ((*print-pretty* nil))
+                           (format stream "[[~d]]" obj)))
+                       1
+                       ppd)
+  (let ((string (write-to-string (make-instance 'ship)
+                                 :pprint-dispatch ppd
+                                 :pretty t)))
+    ;; Do not want to see "#<SHIP {[[decimal-integer]]}>"
+    (assert (not (search "{[[" string)))))
+
 ;;; success
