@@ -19,7 +19,7 @@
   (let ((fun (compile nil
                       `(sb-int:named-lambda test ()
                          (declare (optimize (debug 0) (safety ,safety)
-                                            (sb-c::verify-arg-count 0)))
+                                            (sb-c:verify-arg-count 0)))
                          ,expr))))
     (sb-int:encapsulate 'sb-disassem::add-debugging-hooks 'test
                         (lambda (f &rest args) (declare (ignore f args))))
@@ -185,15 +185,15 @@
       (let ((lines (disassembly-lines 'c)))
         (expect "#<FUNCTION H>" lines))
       (let ((lines (disassembly-lines 'd)))
-        (expect "#<FDEFN F>" lines)
-        (expect "#<FDEFN G>" lines))
-      (handler-bind ((warning #'muffle-warning))
-        (defun g (x) (- x)))
+        (expect "#<FUNCTION H>" lines))
+      (setf (symbol-function 'g) #'+)
       (let ((lines (disassembly-lines 'c)))
-        (expect "#<FDEFN G>" lines)))))
+        (expect "#<FDEFN G>" lines))
+      (let ((lines (disassembly-lines 'd)))
+        (expect "#<FDEFN G>" lines)
+        (expect "#<FUNCTION H>" lines)))))
 
-(with-test (:name :c-call
-            :skipped-on (not :sb-dynamic-core))
+(with-test (:name :c-call)
   (let* ((lines (split-string
                  (with-output-to-string (s)
                    (let ((sb-disassem:*disassem-location-column-width* 0))
@@ -338,12 +338,12 @@
     (try '(cdr obj) '("hi" . 1) '("hi"))))
 
 (with-test (:name :huge-code :skipped-on (not :immobile-code))
-  (sb-vm::allocate-code-object :immobile 4 (* 2 1024 1024)))
+  (sb-vm::allocate-code-object :immobile 0 4 (* 2 1024 1024)))
 
 (defun bbb (x y z)
   ;; I don't want the number of expected comparisons to depend on whether
   ;; the code gets disassembled with versus without the initial segment.
-  (declare (optimize (sb-c::verify-arg-count 0)))
+  (declare (optimize (sb-c:verify-arg-count 0)))
   (if x
       (ecase y
         (-2 'a) (2 'b) (3 'c) (4 (error "no")) (5 'd) (6 'e) (7 'wat) (8 '*) (9 :hi))
@@ -351,7 +351,7 @@
         (#\a :a) (#\b :b) (#\e :c) (#\f :d) (#\g :e) (#\h :f) (t nil))))
 
 (defun try-case-known-fixnum (x)
-  (declare (optimize (sb-c::verify-arg-count 0)))
+  (declare (optimize (sb-c:verify-arg-count 0)))
   (case (the fixnum x)
     (0 :a) (1 :b) (2 :c) (5 :d) (6 :c) (-1 :blah)))
 (defun try-case-maybe-fixnum (x)
@@ -360,11 +360,11 @@
       (0 :a) (1 :b) (2 :c) (5 :d) (6 :c) (-1 :blah))))
 
 (defun try-case-known-char (x)
-  (declare (optimize (sb-c::verify-arg-count 0)))
+  (declare (optimize (sb-c:verify-arg-count 0)))
   (case (the character x)
     (#\a :a) (#\b :b)(#\c :c) (#\d :d) (#\e :e) (#\f :b)))
 (defun try-case-maybe-char (x)
-  (declare (optimize (sb-c::verify-arg-count 0)))
+  (declare (optimize (sb-c:verify-arg-count 0)))
   (when (characterp x)
     (case x (#\a :a) (#\b :b)(#\c :c) (#\d :d) (#\e :e) (#\f :a))))
 
@@ -495,7 +495,7 @@
 (with-test (:name :disassembler-label-jump-table-targets)
   (let* ((f (checked-compile
              '(lambda (x)
-               (declare (optimize (sb-c::verify-arg-count 0)))
+               (declare (optimize (sb-c:verify-arg-count 0)))
                (case (truly-the fixnum x)
                  (0 (a)) (1 (b)) (2 (c)) (3 (d))))
              :allow-style-warnings t))
@@ -671,6 +671,9 @@ sb-vm::(define-vop (cl-user::test)
   ;; Expect to reference the CTYPE layout because %%TYPEP declares its argument
   ;; to be that.  Expect to reference the UNKNOWN-TYPE layout because of an
   ;; explicit call to UNKNOWN-TYPE-P; same for FUN-DESIGNATOR-TYPE-P.
+
+  ;; The other types are referenced from other functions in the code
+  ;; component.
   (let ((names
           (mapcar (lambda (x)
                     (sb-kernel:classoid-name (sb-kernel:layout-classoid x)))
@@ -679,6 +682,10 @@ sb-vm::(define-vop (cl-user::test)
                                   '(sb-kernel:ctype
                                     sb-kernel:unknown-type
                                     sb-kernel:fun-designator-type
+                                    sb-c::abstract-lexenv
+                                    sb-kernel::classoid-cell
+                                    sb-kernel:layout
+                                    sb-kernel:classoid
                                     #-immobile-space null))))))
 
 ;; lp#1857861
@@ -701,3 +708,13 @@ sb-vm::(define-vop (cl-user::test)
 (with-test (:name :thread-local-unbound)
   (let ((c (nth-value 1 (ignore-errors sb-c::*compilation*))))
     (assert (eq (cell-error-name c) 'sb-c::*compilation*))))
+
+#+immobile-code
+(with-test (:name :debug-fun-from-pc-more-robust)
+  (let ((trampoline
+          (sb-di::code-header-from-pc
+           (sb-sys:int-sap (sb-vm::fdefn-raw-addr
+                            (sb-kernel::find-fdefn 'sb-kernel::get-internal-real-time))))))
+    (assert (zerop (sb-kernel:code-n-entries trampoline)))
+    (assert (typep (sb-di::debug-fun-from-pc trampoline 8)
+                   'sb-di::bogus-debug-fun))))
