@@ -31,7 +31,8 @@
                 #:seg-virtual-location #:seg-length #:seg-sap-maker
                 #:map-segment-instructions #:inst-name
                 #:dstate-next-addr #:dstate-cur-offs)
-  (:import-from "SB-X86-64-ASM" #:near-jump-displacement #:mov #:call
+  (:import-from "SB-X86-64-ASM" #:near-jump-displacement
+                #:near-cond-jump-displacement #:mov #:call #:jmp
                 #:get-gpr #:reg-name)
   (:import-from "SB-IMPL" #:package-hashtable #:package-%name
                 #:package-hashtable-cells
@@ -114,6 +115,8 @@
   (linkage-symbols nil)
   (linkage-symbol-usedp nil)
   (linkage-entry-size nil)
+  (new-fixups (make-hash-table))
+  (new-fixup-words-used 0)
   ;; For assembler labels that we want to invent at random
   (label-counter 0)
   (enable-pie nil)
@@ -125,6 +128,16 @@
   (jmp-inst nil :read-only t)
   (pop-inst nil :read-only t))
 
+(defglobal *editcore-ppd*
+  ;; copy no entries for macros/special-operators (flet, etc)
+  (let ((ppd (sb-pretty::make-pprint-dispatch-table nil nil nil)))
+    (set-pprint-dispatch 'string
+                         ;; Write strings without string quotes
+                         (lambda (stream string) (write-string string stream))
+                         0
+                         ppd)
+    ppd))
+
 (defun c-name (lispname core pp-state &optional (prefix ""))
   (when (typep lispname '(string 0))
     (setq lispname "anonymous"))
@@ -146,7 +159,9 @@
                             (concatenate 'string "_" lispname))
                            (t
                             (write-to-string lispname
-                              :pretty t :pprint-dispatch (cdr pp-state)
+                              ;; Printing is a tad faster without a pretty stream
+                              :pretty (not (typep lispname 'core-sym))
+                              :pprint-dispatch *editcore-ppd*
                               ;; FIXME: should be :level 1, however see
                               ;; https://bugs.launchpad.net/sbcl/+bug/1733222
                               :escape t :level 2 :length 5
@@ -411,22 +426,28 @@
 (defun labelize (x) (concatenate 'string label-prefix x))
 
 (defun compute-linkage-symbols (spaces entry-size)
-  (let* ((hashtable (symbol-global-value
-                     (find-target-symbol "SB-SYS" "*LINKAGE-INFO*"
-                                         spaces :physical)))
+  (let* ((linkage-info (symbol-global-value
+                        (find-target-symbol "SB-SYS" "*LINKAGE-INFO*"
+                                            spaces :physical)))
+         (hashtable (car (translate linkage-info spaces)))
          (pairs (target-hash-table-alist hashtable spaces))
          (min (reduce #'min pairs :key #'cdr))
          (max (reduce #'max pairs :key #'cdr))
-         (n (1+ (/ (- max min) entry-size)))
+         (n (1+ (- max min)))
          (vector (make-array n)))
     (dolist (entry pairs vector)
       (let* ((key (undescriptorize (car entry)))
-             (entry-index (/ (- (cdr entry) min) entry-size))
+             (entry-index (- (cdr entry) min))
              (string (labelize (translate (if (consp key) (car (translate key spaces)) key)
                                           spaces))))
         (setf (aref vector entry-index)
               (if (consp key) (list string) string))))))
 
+(defconstant inst-call (find-inst #b11101000 (get-inst-space)))
+(defconstant inst-jmp (find-inst #b11101001 (get-inst-space)))
+(defconstant inst-jmpz (find-inst #x840f (get-inst-space)))
+(defconstant inst-pop (find-inst #x5d (get-inst-space)))
+
 (defun make-core (spaces code-bounds fixedobj-bounds &optional enable-pie)
   (let* ((linkage-bounds
           (make-bounds
@@ -453,10 +474,7 @@
            :linkage-entry-size linkage-entry-size
            :linkage-symbols linkage-symbols
            :linkage-symbol-usedp (make-array (length linkage-symbols) :element-type 'bit)
-           :enable-pie enable-pie
-           :call-inst (find-inst #b11101000 inst-space)
-           :jmp-inst (find-inst #b11101001 inst-space)
-           :pop-inst (find-inst #x5d inst-space))))
+           :enable-pie enable-pie)))
     (let ((package-table
            (symbol-global-value
             (find-target-symbol "SB-KERNEL" "*PACKAGE-NAMES*" spaces :physical)))
@@ -501,21 +519,6 @@
          (acond ((and (< i (length exceptions)) (aref exceptions i))
                  (write-string it stream))
                 (t
-                 #+nil
-                 (let ((len
-                        ;; output-reasonable-integer-in-base is so slow comparated
-                        ;; to printf() that the second-most amount of time spent
-                        ;; writing the asm file occurs in that function.
-                        ;; Unbelievable that we can't do better than that.
-                        (with-pinned-objects (string-buffer fmt)
-                          (alien-funcall
-                           (extern-alien "snprintf"
-                            (function int system-area-pointer unsigned system-area-pointer unsigned))
-                           (vector-sap string-buffer)
-                           (length string-buffer)
-                           (vector-sap fmt)
-                           (sap-ref-word sap (* i n-word-bytes))))))
-                   (write-string string-buffer stream :end len))
                  (write-string "0x" stream)
                  (write (sap-ref-word sap (* i n-word-bytes)) :stream stream)))
          (when (and (= (incf per-line) 16) (< (1+ i) count))
@@ -551,9 +554,6 @@
 (defun list-textual-instructions (sap length core load-addr emit-cfi)
   (let ((dstate (core-dstate core))
         (seg (core-seg core))
-        (call-inst (core-call-inst core))
-        (jmp-inst (core-jmp-inst core))
-        (pop-inst (core-pop-inst core))
         (next-fixup-addr
          (or (car (core-fixup-addrs core)) most-positive-word))
         (list))
@@ -587,30 +587,41 @@
                                       (signed-sap-ref-8 sap (+ offs 2))
                                       (reg-name (get-gpr :qword reg)))))
                    (push (list* (1- (dstate-cur-offs dstate)) 8 "mov" text) list)))
-                ((and (eq (inst-name inst) 'call) ; match "call qword ptr [addr]"
-                      (eql (ldb (byte 24 0) (sap-ref-32 sap offs))
-                           #x2514FF)) ; ModRM+SIB encodes disp32, no base, no index
-                 ;; This form of call instruction is employed for asm routines when
-                 ;; compile-to-memory-space is :AUTO.  If the code were to be loaded
-                 ;; into dynamic space, the offset to the called routine isn't
-                 ;; a (signed-byte 32), so we need the indirection.
-                 (push (list* (dstate-cur-offs dstate) 7 "call*" operand) list))
+                ((let ((bytes (ldb (byte 24 0) (sap-ref-32 sap offs))))
+                   (or (and (eq (inst-name inst) 'call) ; match "{call,jmp} qword ptr [addr]"
+                            (eql bytes #x2514FF)) ; ModRM+SIB encodes disp32, no base, no index
+                       (and (eq (inst-name inst) 'jmp)
+                            (eql bytes #x2524FF))))
+                 (let ((new-opcode (ecase (sap-ref-8 sap (1+ offs))
+                                     (#x14 "call *")
+                                     (#x24 "jmp *"))))
+                   ;; This instruction form is employed for asm routines when
+                   ;; compile-to-memory-space is :AUTO.  If the code were to be loaded
+                   ;; into dynamic space, the offset to the called routine isn't
+                   ;; a (signed-byte 32), so we need the indirection.
+                   (push (list* (dstate-cur-offs dstate) 7 new-opcode operand) list)))
                 (t
                  (bug "Can't reverse-engineer fixup: ~s ~x"
-                      (inst-name inst) (sap-ref-32 sap offs))))))
+                      (inst-name inst) (sap-ref-64 sap offs))))))
           (pop (core-fixup-addrs core))
           (setq next-fixup-addr (or (car (core-fixup-addrs core)) most-positive-word)))
-         ((or (eq inst jmp-inst) (eq inst call-inst))
+         ((or (eq inst inst-jmp) (eq inst inst-call))
           (let ((target-addr (+ (near-jump-displacement dchunk dstate)
                                 (dstate-next-addr dstate))))
             (when (or (in-bounds-p target-addr (core-fixedobj-bounds core))
                       (in-bounds-p target-addr (core-linkage-bounds core)))
               (push (list* (dstate-cur-offs dstate)
                            5 ; length
-                           (if (eq inst call-inst) "call" "jmp")
+                           (if (eq inst inst-call) "call" "jmp")
                            target-addr)
                     list))))
-         ((and (eq inst pop-inst) (eq (logand dchunk #xFF) #x5D))
+         ((eq inst inst-jmpz)
+          (let ((target-addr (+ (near-cond-jump-displacement dchunk dstate)
+                                (dstate-next-addr dstate))))
+            (when (in-bounds-p target-addr (core-linkage-bounds core))
+              (push (list* (dstate-cur-offs dstate) 6 "je" target-addr)
+                    list))))
+         ((and (eq inst inst-pop) (eq (logand dchunk #xFF) #x5D))
           (push (list* (dstate-cur-offs dstate) 1 "pop" "%rbp") list))))
      seg
      dstate
@@ -648,6 +659,8 @@
 ;;; This is tricky to fix because while we can relativize the CFA to the
 ;;; known frame size, we can't do that based only on a disassembly.
 
+;;; Return the list of locations which must be added to code-fixups
+;;; in the event that heap relocation occurs on image restart.
 (defun emit-lisp-function (paddr vaddr count stream emit-cfi core &optional labels)
   (when emit-cfi
     (format stream " .cfi_startproc~%"))
@@ -658,6 +671,7 @@
          (merge 'list labels
                 (list-textual-instructions (int-sap paddr) count core vaddr emit-cfi)
                 #'< :key #'car))
+        (extra-fixup-locs)
         (ptr paddr))
     (symbol-macrolet ((cur-offset (- ptr paddr)))
       (loop
@@ -703,13 +717,16 @@
           ;; to see them where they belong in the instruction stream]
           (when (and instructions (= (caar instructions) cur-offset))
             (destructuring-bind (length opcode . operand) (cdr (pop instructions))
-              (when (cond ((member opcode '("jmp" "call") :test #'string=)
+              (when (cond ((member opcode '("jmp" "je" "call") :test #'string=)
                            (when (in-bounds-p operand (core-linkage-bounds core))
                              (let ((entry-index
                                     (/ (- operand (bounds-low (core-linkage-bounds core)))
                                        (core-linkage-entry-size core))))
                                (setf (bit (core-linkage-symbol-usedp core) entry-index) 1
                                      operand (aref (core-linkage-symbols core) entry-index))))
+                           (when (and (integerp operand)
+                                      (in-bounds-p operand (core-fixedobj-bounds core)))
+                             (push (+ vaddr cur-offset) extra-fixup-locs))
                            (format stream " ~A ~:[0x~X~;~a~:[~;@PLT~]~]~%"
                                    opcode (stringp operand) operand
                                    (core-enable-pie core)))
@@ -725,18 +742,20 @@
                            ;; the so-called "operand" is the entire instruction
                            (write-string operand stream)
                            (terpri stream))
-                          ((string= opcode "call*")
+                          ((or (string= opcode "call *") (string= opcode "jmp *"))
                            ;; Indirect call - since the code is in immobile space,
                            ;; we could render this as a 2-byte NOP followed by a direct
                            ;; call. For simplicity I'm leaving it exactly as it was.
-                           (format stream " call *(CS+0x~x)~%"
+                           (format stream " ~A(CS+0x~x)~%"
+                                   opcode ; contains a "*" as needed for the syntax
                                    (- operand (bounds-low (core-code-bounds core)))))
                           (t))
                 (bug "Random annotated opcode ~S" opcode))
               (incf ptr length)))
-          (when (= cur-offset count) (return))))))
-  (when emit-cfi
-    (format stream " .cfi_endproc~%")))
+          (when (= cur-offset count) (return)))))
+    (when emit-cfi
+      (format stream " .cfi_endproc~%"))
+    extra-fixup-locs))
 
 ;;; Examine CODE, returning a list of lists describing how to emit
 ;;; the contents into the assembly file.
@@ -811,6 +830,7 @@
          (text (sap-int text-sap))
          ;; Like CODE-INSTRUCTIONS, but where the text virtually was
          (text-vaddr (+ vaddr (* (code-header-words code) n-word-bytes)))
+         (additional-relative-fixups)
          (max-end 0))
     ;; There is *always* at least 1 word of unboxed data now
     (aver (eq (caar ranges) :data))
@@ -845,8 +865,11 @@
         ;; Pass the current physical address at which to disassemble,
         ;; the notional core address (which changes after linker relocation),
         ;; and the length.
-        (emit-lisp-function (+ text start) (+ text-vaddr start) (- end start)
-                            output emit-cfi core)
+        (let ((new-relative-fixups
+               (emit-lisp-function (+ text start) (+ text-vaddr start) (- end start)
+                                   output emit-cfi core)))
+          (setq additional-relative-fixups
+                (nconc new-relative-fixups additional-relative-fixups)))
         (cond ((not ranges) (return))
               ((eq (caar ranges) :pad)
                (format output " .byte ~{0x~x~^,~}~%"
@@ -862,10 +885,40 @@
             (loop for i from max-end
                   below (- (code-object-size code)
                            (* (code-header-words code) n-word-bytes))
-                  collect (sap-ref-8 text-sap i)))))
+                  collect (sap-ref-8 text-sap i)))
+    (when additional-relative-fixups
+      (binding* ((existing-fixups (sb-vm::%code-fixups code))
+                 ((absolute relative)
+                  (sb-c::unpack-code-fixup-locs
+                   (if (fixnump existing-fixups)
+                       existing-fixups
+                       (translate existing-fixups spaces))))
+                 (new-sorted
+                  (sort (mapcar (lambda (x)
+                                  ;; compute offset of the fixup from CODE-INSTRUCTIONS.
+                                  ;; X is the location of the CALL instruction,
+                                  ;; 1+ is the location of the fixup.
+                                  (- (1+ x)
+                                     (+ vaddr (ash (code-header-words code)
+                                                   sb-vm:word-shift))))
+                                additional-relative-fixups)
+                        #'<)))
+        (sb-c::pack-code-fixup-locs
+         absolute
+         (merge 'list relative new-sorted #'<))))))
 
 (defconstant +gf-name-slot+ 5)
 
+(defun output-bignum (label bignum stream)
+  (let ((nwords (sb-bignum:%bignum-length bignum)))
+    (format stream "~@[~a:~] .quad 0x~x"
+            label (logior (ash nwords 8) sb-vm:bignum-widetag))
+    (dotimes (i nwords)
+      (format stream ",0x~x" (sb-bignum:%bignum-ref bignum i)))
+    (when (evenp nwords) ; pad
+      (format stream ",0"))
+    (format stream "~%")))
+
 (defun write-preamble (output)
   (format output " .text~% .file \"sbcl.core\"
 ~:[~; .macro .size sym size # ignore
@@ -896,23 +949,18 @@
           (core (make-core spaces code-bounds fixedobj-bounds enable-pie))
           (code-addr (bounds-low code-bounds))
           (total-code-size 0)
-          (pp-state (cons (make-hash-table :test 'equal)
-                          ;; copy no entries for macros/special-operators (flet, etc)
-                          (sb-pretty::make-pprint-dispatch-table nil nil nil)))
+          (pp-state (cons (make-hash-table :test 'equal) nil))
           (prev-namestring "")
           (n-linker-relocs 0)
           (seen-fdefns nil)
           (seen-trampolines nil)
           (seen-gfs nil)
+          (temp-output (make-string-output-stream :element-type 'base-char))
           end-loc)
-  (set-pprint-dispatch 'string
-                       ;; Write strings without string quotes
-                       (lambda (stream string) (write-string string stream))
-                       0
-                       (cdr pp-state))
   (labels ((dumpwords (sap count stream &optional (exceptions #()) logical-addr)
              (aver (sap>= sap (car spaces)))
-             ;; Make intra-code-space pointers computed at link time
+             ;; Add any new "header exceptions" that cause intra-code-space pointers
+             ;; to be computed at link time
              (dotimes (i (if logical-addr count 0))
                (unless (and (< i (length exceptions)) (svref exceptions i))
                  (let ((word (sap-ref-word sap (* i n-word-bytes))))
@@ -982,13 +1030,16 @@
                                    (- obj-size (* header-len n-word-bytes))
                                    (1+ end-offs))
                                start-offs)))
-              (format output " lasmsym ~(\"~a\"~), ~d~%" name nbytes)
-              (emit-lisp-function (+ (sap-int (code-instructions code-component))
-                                     start-offs)
-                                  (+ code-addr
-                                     (ash (code-header-words code-component) word-shift)
-                                     start-offs)
-                                  nbytes output nil core)))
+                (format output " lasmsym ~(\"~a\"~), ~d~%" name nbytes)
+                (let ((fixups
+                       (emit-lisp-function
+                        (+ (sap-int (code-instructions code-component))
+                           start-offs)
+                        (+ code-addr
+                           (ash (code-header-words code-component) word-shift)
+                           start-offs)
+                        nbytes output nil core)))
+                  (aver (null fixups)))))
             (when (endp list) (return)))
           (incf code-addr obj-size)
           (setf total-code-size obj-size))))
@@ -1034,10 +1085,30 @@
                 (format output "#x~x:~%" code-addr)
                 ;; Emit symbols before the code header data, because the symbols
                 ;; refer to "." (the current PC) which is the base of the object.
-                (let ((base (emit-symbols (code-symbols code core) core pp-state output)))
+                (let* ((base (emit-symbols (code-symbols code core) core pp-state output))
+                       (altered-fixups
+                        (emit-funs code code-addr core #'dumpwords temp-output base emit-cfi))
+                       (header-exceptions (vector nil nil nil nil))
+                       (fixups-ptr))
+                  (when altered-fixups
+                    (setf (aref header-exceptions 3)
+                          (cond ((fixnump altered-fixups)
+                                 (format nil "0x~x" (ash altered-fixups sb-vm:n-fixnum-tag-bits)))
+                                (t
+                                 (let ((ht (core-new-fixups core)))
+                                   (setq fixups-ptr (gethash altered-fixups ht))
+                                   (unless fixups-ptr
+                                     (setq fixups-ptr (ash (core-new-fixup-words-used core)
+                                                           sb-vm:word-shift))
+                                     (setf (gethash altered-fixups ht) fixups-ptr)
+                                     (incf (core-new-fixup-words-used core)
+                                           (align-up (1+ (sb-bignum:%bignum-length altered-fixups)) 2))))
+                                 ;; tag the pointer properly for a bignum
+                                 (format nil "lisp_fixups+0x~x"
+                                         (logior fixups-ptr sb-vm:other-pointer-lowtag))))))
                   (dumpwords (int-sap code-physaddr)
-                             (code-header-words code) output #() code-addr)
-                  (emit-funs code code-addr core #'dumpwords output base emit-cfi))))
+                             (code-header-words code) output header-exceptions code-addr)
+                  (write-string (get-output-stream-string temp-output) output))))
              ((functionp (%code-debug-info code))
               (unless seen-trampolines
                 (setq seen-trampolines t)
@@ -1491,8 +1562,7 @@
                              (+ core-offs n-word-bytes)
                              word)))
               (when (eq widetag funcallable-instance-widetag)
-                (let ((layout (truly-the layout
-                               (translate (%funcallable-instance-layout obj) spaces))))
+                (let ((layout (truly-the layout (translate (%fun-layout obj) spaces))))
                   (unless (fixnump (layout-bitmap layout))
                     (error "Can't process bignum bitmap"))
                   (let ((bitmap (layout-bitmap layout)))
@@ -1778,6 +1848,10 @@
             (copy-bytes input output pte-nbytes)) ; Copy PTEs from input
           (let ((core (write-assembler-text map asm-file enable-pie))
                 (emit-all-c-symbols t))
+            (format asm-file " .section .rodata~% .p2align 4~%lisp_fixups:~%")
+            ;; Sort the hash-table in emit order.
+            (dolist (x (sort (%hash-table-alist (core-new-fixups core)) #'< :key #'cdr))
+              (output-bignum nil (car x) asm-file))
             (format asm-file (if (member :darwin *features*)
                                  "~% .data~%"
                                  "~% .section .rodata~%"))
